/****************************************************************/
/****************************************************************/
/*                 SOUND EDITOR                          
/*
/*			FILE:	soundedit.cpp
/*			AUTHOR: Ryan C. Clausing
/*			DATE:   Spring 2002
/*
/****************************************************************/
/****************************************************************/

#include <afxwin.h>
#include <windows.h>
#include <windowsx.h>
#include <afxcmn.h>
#include <afxdlgs.h>
#include <mmsystem.h>
#include <mmreg.h>
#include <fstream.h>
#include <stdlib.h>
#include <math.h>
#include "soundedit.h"
#include "all_ids.h" 

#define DARKGRAY RGB(100,100,100)
#define RED RGB(255,0,0)
#define DARKBLUE RGB(0,0,128)
#define BLACK RGB(0,0,0)
#define WAVECOLOR RGB(44, 255, 183)

#define BOTTOM 82
#define CONTROLBARTOP 42
#define TOP 18
#define LEFT 10
#define RIGHT 20

#define PI 3.141592654

CMyApp myApp;
CMainWindow *mw;

HANDLE CreateMyThread(CString x);

/* make these global to make life easier */
WAVEFORMATEX WaveFormat;
HMMIO		HMmio;
MMCKINFO    MmckinfoSubchunk;
DWORD		WaveDataPlayed;
HWAVEOUT	HWaveOut;;

BOOL CMyApp::InitInstance ()
{
	m_pMainWnd = new CMainWindow;
    m_pMainWnd->ShowWindow (m_nCmdShow);
    m_pMainWnd->UpdateWindow ();
	if ( !m_pMainWnd->SetTimer( 1, 10, NULL )) {
		return FALSE; 
	}
	return TRUE;
}


CMainWindow::CMainWindow (){
	int x = GetSystemMetrics(SM_CXSCREEN);
	int y = GetSystemMetrics(SM_CYSCREEN);
	int x1, x2, y1, y2;

	x1 = x - (x / 10);
	y1 = y - (y / 10);
	x2 = (x - x1) / 2;
	y2 = (y - y1) / 2;

	winrect = CRect(x2, y2, x1, y1);
	Create(0, "Sound Edit 0.01", WS_OVERLAPPEDWINDOW, winrect, NULL, "File");	

	InitVars();
	SetVolume();
	DrawLayout();

	return;
}

/*************************************************************/
/*************************************************************/
/*
/*  Parameters: str - not used in my code
/*
/*  Returns: 0 - if no error
/*
/*  Description: 
/*		Gets called when the main window is created.  This
/*		function creates my toolbar, the toolbar buttons, and 
/*      also loads the images for the buttons 
/*
/*************************************************************/
/*************************************************************/
int CMainWindow::OnCreate(LPCREATESTRUCT str) {
	CRect brect;
	GetClientRect(&arect);

	brect.SetRect(arect.left, arect.bottom - CONTROLBARTOP, arect.right, arect.bottom);

	CWinApp *pApp = AfxGetApp();
	
	ToolBar.Create(WS_CHILD | WS_VISIBLE | CCS_BOTTOM | CCS_NOMOVEY, brect, this, IDC_TOOLBARCTRL);
    ToolBar.SetIndent(2);

	EnabledImageList.Create(16, 16, ILC_COLOR | ILC_MASK, 8, 8);
	EnabledImageList.Add(pApp->LoadIcon(ICO_ENABLED_PLAY));
	EnabledImageList.Add(pApp->LoadIcon(ICO_ENABLED_PAUSE));
	EnabledImageList.Add(pApp->LoadIcon(ICO_ENABLED_GOTOBEGIN));
	EnabledImageList.Add(pApp->LoadIcon(ICO_ENABLED_REWIND));
	EnabledImageList.Add(pApp->LoadIcon(ICO_ENABLED_FORWARD));
	EnabledImageList.Add(pApp->LoadIcon(ICO_ENABLED_GOTOEND));
	EnabledImageList.Add(pApp->LoadIcon(ICO_ENABLED_STOP));
	EnabledImageList.Add(pApp->LoadIcon(ICO_ENABLED_RECORD));

	DisabledImageList.Create(16, 16, ILC_COLOR | ILC_MASK, 8, 8);
	DisabledImageList.Add(pApp->LoadIcon(ICO_DISABLED_PLAY));
	DisabledImageList.Add(pApp->LoadIcon(ICO_DISABLED_PAUSE));
	DisabledImageList.Add(pApp->LoadIcon(ICO_DISABLED_GOTOBEGIN));
	DisabledImageList.Add(pApp->LoadIcon(ICO_DISABLED_REWIND));
	DisabledImageList.Add(pApp->LoadIcon(ICO_DISABLED_FORWARD));
	DisabledImageList.Add(pApp->LoadIcon(ICO_DISABLED_GOTOEND));
	DisabledImageList.Add(pApp->LoadIcon(ICO_DISABLED_STOP));
	DisabledImageList.Add(pApp->LoadIcon(ICO_DISABLED_RECORD));

	HotImageList.Create(16, 16, ILC_COLOR | ILC_MASK, 8, 8);
	HotImageList.Add(pApp->LoadIcon(ICO_HOT_PLAY));
	HotImageList.Add(pApp->LoadIcon(ICO_HOT_PAUSE));
	HotImageList.Add(pApp->LoadIcon(ICO_HOT_GOTOBEGIN));
	HotImageList.Add(pApp->LoadIcon(ICO_HOT_REWIND));
	HotImageList.Add(pApp->LoadIcon(ICO_HOT_FORWARD));
	HotImageList.Add(pApp->LoadIcon(ICO_HOT_GOTOEND));
	HotImageList.Add(pApp->LoadIcon(ICO_HOT_STOP));
	HotImageList.Add(pApp->LoadIcon(ICO_HOT_RECORD));

 	ToolBar.SetImageList(&EnabledImageList);
	ToolBar.SetHotImageList(&HotImageList);
	ToolBar.SetDisabledImageList(&DisabledImageList);

	ToolBar.AddStrings("Play");
	ToolBar.AddStrings("Pause");
	ToolBar.AddStrings("Go to Start");
	ToolBar.AddStrings("Rewind");
	ToolBar.AddStrings("Fast Fwd");
	ToolBar.AddStrings("Go to End");
	ToolBar.AddStrings("Stop");
	ToolBar.AddStrings("Record");


	AButton.fsState = TBSTATE_ENABLED;
	AButton.fsStyle = TBSTYLE_BUTTON;
	AButton.dwData = NULL;

	AButton.iBitmap = 0;
	AButton.iString = 0;
	AButton.idCommand = IDC_PLAY;
	ToolBar.InsertButton(0, &AButton);

	AButton.iBitmap = 1;
	AButton.iString = 1;
	AButton.idCommand = IDC_PAUSE;
	ToolBar.InsertButton(1, &AButton);

	AButton.iBitmap = 2;
	AButton.iString = 2;
	AButton.idCommand = IDC_GOTOSTART;
	ToolBar.InsertButton(2, &AButton);

	AButton.iBitmap = 3;
	AButton.iString = 3;
	AButton.idCommand = IDC_REWIND;
	ToolBar.InsertButton(3, &AButton);

	AButton.iBitmap = 4;
	AButton.iString = 4;
	AButton.idCommand = IDC_FASTFWD;
	ToolBar.InsertButton(4, &AButton);

	AButton.iBitmap = 5;
	AButton.iString = 5;
	AButton.idCommand = IDC_GOTOEND;
	ToolBar.InsertButton(5, &AButton);

	AButton.iBitmap = 6;
	AButton.iString = 6;
	AButton.idCommand = IDC_STOP;
	ToolBar.InsertButton(6, &AButton);

	AButton.iBitmap = 7;
	AButton.iString = 7;
	AButton.idCommand = IDC_RECORD;
	ToolBar.InsertButton(7, &AButton);	


	///////enable and disable buttons///////
	ToolBar.EnableButton(IDC_PLAY,FALSE);
	ToolBar.EnableButton(IDC_REWIND,FALSE);
	ToolBar.EnableButton(IDC_FASTFWD,FALSE);
	ToolBar.EnableButton(IDC_STOP,FALSE);
	ToolBar.EnableButton(IDC_GOTOEND,FALSE);
	ToolBar.EnableButton(IDC_GOTOSTART,FALSE);
	ToolBar.EnableButton(IDC_PAUSE,FALSE);
	////////////////////////////////////////
	mw  = this;
	return 0;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function does all my initialization of variables
/*		that I use throughout the program.  It also gets the 
/*		current volume level of the wave out on the sound card.
/*
/*************************************************************/
/*************************************************************/
void CMainWindow::InitVars() {
	//get starting volume from the sound card
	waveOutOpen(&HWaveOut, WAVE_MAPPER, &WaveFormat, NULL, 0, CALLBACK_NULL);
	waveOutGetVolume(HWaveOut, &dwVolume);
	waveOutClose(HWaveOut);

	//set the top nodes of linked lists = NULL
	wavefile.wd.rightchannel = NULL;
	wavefile.wd.leftchannel = NULL;
	posline = NULL;

	//set vars to initial values
	wavefile.numsamples = 0;
	startselection = -1;
	endselection = -1;
	nextmark = -1;
	lastmark = -1;
	lensilence = 0;
	PlayStart = PlayEnd = -1;
	linepos = -1;
	volumelevel = 0;
	pw.count = 1;
	SizeOfClipboardData = 0;
	FileSelStart = FileSelEnd = -1;
	FileDataStartPos = CurrentFileDataPos = -1;

	//set bools to initial values
	fileopen = processing = false;
	lbuttondown = false;
	shiftkey = controlkey = false;
	stopped = true;
	playing = false;
	paused = false;
	recording = loop = changedsincesaved = working = false;
	rightchannel = leftchannel = true;

	//get the color of the menus (for the border around the file display
	GrayColor = (COLORREF)GetSysColor(COLOR_MENU);

	//set all strings to ""
	wavefile.modfile = "";
	wavefile.workfile = "";
	wavefile.tmpfile = "";
	wavefile.undofile = "";
	wavefile.clipfile = "";
	statusstate = "";
	statustext = "";
	laststatusstate = "";
	laststatustext = "";
	
	//calculate all window rectangles (makes it easier to draw having set rectangles)
	GetClientRect(&winrect);
	toprect.top = winrect.top;
	toprect.bottom = winrect.top + TOP;
	toprect.left = winrect.left + LEFT;
	toprect.right = winrect.right - RIGHT;

	//tophalf is the top half of the wave display (left channel)
	tophalf.top = winrect.top + TOP + 1;
	tophalf.bottom = (((winrect.bottom - BOTTOM) - (winrect.top + TOP)) / 2) + TOP - 2;
	tophalf.left = winrect.left + LEFT;
	tophalf.right = winrect.right - RIGHT;

	//bottomhalf is the bottom half of the wave display (right channel)
	bottomhalf.top = (((winrect.bottom - BOTTOM) - (winrect.top + TOP)) / 2) + TOP + 2;
	bottomhalf.bottom = winrect.bottom - BOTTOM;
	bottomhalf.left = winrect.left + LEFT;
	bottomhalf.right = winrect.right - RIGHT;

	//midline is the divider between tophalf and bottomhalf
	midline = tophalf.bottom + 1;

	//statusbox is the space above the ToolBar where the status of the program is shown
	statusbox.top = bottomhalf.bottom;
	statusbox.left = winrect.left + LEFT;
	statusbox.right = winrect.right - RIGHT;
	statusbox.bottom = winrect.bottom - CONTROLBARTOP;

	//qlinetop and qlinebottom are the mid-lines in the tophalf and bottomhalf rects
	qlinetop = ((tophalf.bottom - tophalf.top) / 2) + TOP; 
	qlinebottom = ((bottomhalf.bottom - bottomhalf.top) / 2) + tophalf.bottom + 4;
	return;
}

////////////////////////////////////////////////////////////////////////////
/////////////////////  CLEANUP FUNCTIONS ///////////////////////////////////
////////////////////////////////////////////////////////////////////////////

/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		Deletes the nodes in the linked list of the Pos Line
/*		
/*************************************************************/
/*************************************************************/
void CMainWindow::DeletePosLineList() {
	Pixel *x, *y;
	x = posline;
	//delete all nodes
	while (x != NULL) {
		y = x->link;
		delete x;
		x = y;
	}
	posline = NULL;
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description: 
/*		Deletes all the nodes in the wave data list.
/*
/*************************************************************/
/*************************************************************/
void CMainWindow::DeleteWaveDataLists() {
	MyWaveDataNode * x, * y;
	x = wavefile.wd.leftchannel;
	//delete all left channel nodes
	while (x != NULL) {
		y = x->link;
		delete x;
		x = y;
	}
	
	x = wavefile.wd.rightchannel;
	//delete all right channel nodes
	while (x != NULL) {
		y = x->link;
		delete x;
		x = y;
	}
	wavefile.wd.leftchannel = NULL;
	wavefile.wd.rightchannel = NULL;
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description: 
/*		I override this fcn to check if the user wants to save
/*		changes to the file if any have been made
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnClose() {
	//call onclosefile to check if the user wants to save changes
	
	if (!OnCloseFile()) {
		return;
	}
	KillTimer(1);
	delete this;
}

////////////////////////////////////////////////////////////////////////////
/////////////////// END CLEANUP FUNCTIONS //////////////////////////////////
////////////////////////////////////////////////////////////////////////////


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		Sets the volume of both channels to a value that my 
/*		program will recognize and sets the right and left
/*		channels on or off (dependong on how the user sets
/*		them.  Both channels start as being on.
/*
/*************************************************************/
/*************************************************************/
void CMainWindow::SetVolume() {
	//open waveout device
	waveOutOpen(&HWaveOut, WAVE_MAPPER, &WaveFormat, NULL, 0, CALLBACK_NULL);
	DWORD dw = 0;
	//set the volume accordingly depending on the value of dwVolume
	if (dwVolume == 0x00000000) {
		dw = 0x00000000;
		waveOutSetVolume(HWaveOut, dw);
		volumelevel = 0;
	} else if ((dwVolume <= 0x22222222) && (dwVolume > 0x00000000)) {
		dwVolume = 0x11111111;
		if (rightchannel && leftchannel) {
			dw = dwVolume;
		} else if (leftchannel) {
			dw = 0x00001111;
		} else if (rightchannel) {
			dw = 0x11110000;
		}
		waveOutSetVolume(HWaveOut, dw);
		volumelevel = 1;
	} else if ((dwVolume <= 0x44444444) && (dwVolume > 0x22222222)) {
		dwVolume = 0x33333333;
		if (rightchannel && leftchannel) {
			dw = dwVolume;
		} else if (leftchannel) {
			dw = 0x00003333;
		} else if (rightchannel) {
			dw = 0x33330000;
		}
		waveOutSetVolume(HWaveOut, dw);
		volumelevel = 2;
	} else if ((dwVolume <= 0x66666666) && (dwVolume > 0x44444444)) {
		dwVolume = 0x55555555;
		if (rightchannel && leftchannel) {
			dw = dwVolume;
		} else if (leftchannel) {
			dw = 0x00005555;
		} else if (rightchannel) {
			dw = 0x55550000;
		}
		waveOutSetVolume(HWaveOut, dw);
		volumelevel = 3;
	} else if ((dwVolume <= 0x88888888) && (dwVolume > 0x66666666)) {
		dwVolume = 0x77777777;
		if (rightchannel && leftchannel) {
			dw = dwVolume;
		} else if (leftchannel) {
			dw = 0x00007777;
		} else if (rightchannel) {
			dw = 0x77770000;
		}
		waveOutSetVolume(HWaveOut, dw);
		volumelevel = 4;
	} else if ((dwVolume <= 0xAAAAAAAA) && (dwVolume > 0x88888888)) {
		dwVolume = 0x99999999;
		if (rightchannel && leftchannel) {
			dw = dwVolume;
		} else if (leftchannel) {
			dw = 0x00009999;
		} else if (rightchannel) {
			dw = 0x99990000;
		}
		waveOutSetVolume(HWaveOut, dw);
		volumelevel = 5;
	} else if ((dwVolume <= 0xCCCCCCCC) && (dwVolume > 0xAAAAAAAA)) {
		dwVolume = 0xBBBBBBBB;
		if (rightchannel && leftchannel) {
			dw = dwVolume;
		} else if (leftchannel) {
			dw = 0x0000BBBB;
		} else if (rightchannel) {
			dw = 0xBBBB0000;
		}
		waveOutSetVolume(HWaveOut, dw);
		volumelevel = 6;
	} else if ((dwVolume <= 0xEEEEEEEE) && (dwVolume > 0xCCCCCCCC)) {
		dwVolume = 0xDDDDDDDD;
		if (rightchannel && leftchannel) {
			dw = dwVolume;
		} else if (leftchannel) {
			dw = 0x0000DDDD;
		} else if (rightchannel) {
			dw = 0xDDDD0000;
		}
		waveOutSetVolume(HWaveOut, dw);
		volumelevel = 7;
	} else if ((dwVolume <= 0xFFFFFFFF) && (dwVolume > 0xEEEEEEEE)) {
		dwVolume = 0xFFFFFFFF;
		if (rightchannel && leftchannel) {
			dw = dwVolume;
		} else if (leftchannel) {
			dw = 0x0000FFFF;
		} else if (rightchannel) {
			dw = 0xFFFF0000;
		}
		waveOutSetVolume(HWaveOut, dw);
		volumelevel = 8;
	}
	//close wave out device
	waveOutClose(HWaveOut);

	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: nChar - the value of the key pressed
/*				nRepCnt - if > 1, the key is being held down
/*				nFlags - flags (not used)
/*
/*  Returns: None
/*
/*  Description: 
/*		This function is used to catch keystrokes for keyboard
/*		shortcuts.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) {
	//itoa(nChar, buff, 10);
	//MessageBox(buff);

	//keyboard shortcuts
	if (nChar == 37) { // left arrow
		OnRewind();
	} else if (nChar == 39) { // right arrow
		OnFastFwd();
	} else if (nChar == 38) { // up arrow
		OnVolumeUp();
	} else if (nChar == 40) { // downarrow
		OnVolumeDown();
	} else if ((nChar == 16) && (nRepCnt == 1)) { // shift key
		shiftkey = true;
	} else if ((nChar == 17) && (nRepCnt == 1)) { // control key
		controlkey = true;
	} else if ((nChar == 90) && (controlkey) && (nRepCnt == 1)) {
		OnUndo();
	} else if ((nChar == 67) && (controlkey) && (nRepCnt == 1)) {
		OnCopy();
	} else if ((nChar == 88) && (controlkey) && (nRepCnt == 1)) {
		OnCut();
	} else if ((nChar == 86) && (controlkey) && (nRepCnt == 1)) {
		OnPaste();
	} else if ((nChar == 83) && (controlkey) && (nRepCnt == 1)) {
		OnSave();
	} else if ((nChar == 46) && (nRepCnt == 1)) {
		OnClear();
	} else if ((nChar == 82) && (controlkey) && (nRepCnt == 1)) {
		OnReverse();
	} else if ((nChar == 65) && (controlkey) && (nRepCnt == 1)) {
		OnSelAll();
	} else if ((nChar == 79) && (controlkey) && (nRepCnt == 1)) {
		OnOpen();
	} else if ((nChar == 73) && (controlkey) && (nRepCnt == 1)) {
		OnInfo();
	} else if ((nChar == 76) && (controlkey) && (nRepCnt == 1)) {
		OnLoopPlayback();
	}
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: nChar - the value of the key pressed
/*				nRepCnt - determines how long it's been held down
/*				nFlags - flags (not used)
/*
/*  Returns: None
/*
/*  Description:
/*		Sets the shiftkey or controlkey bools to false (these
/*		are needed for keyboard shortcuts.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) {
	//set keys to not pressed
	if (nChar == 16) { // shift key up
		shiftkey = false;
	} else if (nChar == 17) { // ctrl key up
		controlkey = false;
	}
}


//////////////////////////////////////////////////////////////////////////////
/////////// SELECTION FUNCTIONS //////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		Redraws the pixels where the dotted posline was.
/*
/*************************************************************/
/*************************************************************/
void CMainWindow::RestoreLine() {
	//draw the pixels back where it should be
	Pixel *p;
	int c = 0;
	CClientDC dc(this);
	int x = (int)((float)linepos * (float)(tophalf.right - tophalf.left)) + LEFT;
	p = posline;
	while (p != NULL) {
		dc.SetPixel(x, tophalf.top + c, p->color);
		p = p->link;
		c++;
	}
	DeletePosLineList();
	posline = NULL;
	DeleteDC(dc);
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		Saves the pixels where the dotted posline will be drawn
/*
/*************************************************************/
/*************************************************************/
void CMainWindow::SaveLine() {
	Pixel *p, *n;
	p = new Pixel;
	CClientDC dc(this);
	int totalpix = 0;
	//find the position on the window
	int x = (int)((float)linepos * (float)(tophalf.right - tophalf.left)) + LEFT;
	totalpix = bottomhalf.bottom - tophalf.top;
	p->color = dc.GetPixel(x, tophalf.top);
	p->link = NULL;
	posline = p;
	//save the pixels
	for (int i = 1; i < totalpix; i++) {
		n = new Pixel;
		n->color = dc.GetPixel(x, tophalf.top + i);
		n->link = NULL;
		p->link = n;
		p = p->link;
	}
	DeleteDC(dc);
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description: 
/*		Draws the dotted posline
/*
/*************************************************************/
/*************************************************************/
void CMainWindow::DrawLine() {
	Pixel *p;
	p = new Pixel;
	CClientDC dc(this);
	int totalpix = 0;
	int f = 0;
	int i = 0;
	//get the position for the line
	int x = (int)((float)linepos * (float)(tophalf.right - tophalf.left)) + LEFT;
	totalpix = bottomhalf.bottom - tophalf.top;
	//draw the dotted line
	while (i < totalpix) { 
		for (int b = 0; b < 2; b++) {
			i++;
		}
		if (i < totalpix) {
			dc.SetPixel(x, tophalf.top + i, RGB(255,255,0));
			i++;
			if (i < totalpix) {
				dc.SetPixel(x, tophalf.top + i, RGB(0,0,0));
				i++;
			} else {
				break;
			}
		} else {
			break;
		}
	}
	DeleteDC(dc);
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: start - start of selection (from 0 to 1)
/*				end - end of selection (from 0 to 1)
/*
/*  Returns: None
/*
/*  Description:
/*		Draws the selection on the screen - the seleciton is 
/*		drawn by inverting the rectangle of the selected region
/*
/*************************************************************/
/*************************************************************/
void CMainWindow::DrawSelection(float start, float end) {
	CRect r;
	CClientDC dc(this);
	int selstart, selend;
	selstart = selend = 0;

	float t = 0;
	if (start > end) {
		t = start;
		start = end;
		end = t;
	}
	//get start and end positions on the window for the selection
	selstart = (int)((float)start * (float)(tophalf.right - tophalf.left)) + LEFT;
	selend = (int)((float)end * (float)(tophalf.right - tophalf.left )) + LEFT;
	if (selstart == 0) {
		selstart = tophalf.left;
	} 
	if (selend == 1) {
		selend = tophalf.right;
	}
	//draw the selection
	if (selstart != selend) {
		r.top = tophalf.top;
		r.bottom = bottomhalf.bottom;
		r.left = selstart;
		r.right = selend;
		dc.InvertRect(r);
	}
	DeleteDC(dc);
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		Calculates the coresponding positions on the file for
/*		the selection start and selection end
/*
/*************************************************************/
/*************************************************************/
void CMainWindow::CalculateFilePositions() {
	if (!fileopen) return;

	float start, end;
	start = startselection;
	end = endselection;

	float t = 0;
	if (start > end) {
		t = start;
		start = end;
		end = t;
	}
	//find the coresponding file positions 
	FileSelStart = (int)((float)start * (float)wavefile.datasize);
	FileSelEnd = (int)((float)end * (float)wavefile.datasize);

	//go to the nearest sample
	while ((FileSelStart % WaveFormat.nBlockAlign) != 0) {
		FileSelStart -= 1;
	}
	while ((FileSelEnd % WaveFormat.nBlockAlign) != 0) {
		FileSelEnd += 1;
	}
	FileSelStart += FileDataStartPos;
	FileSelEnd += FileDataStartPos;
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: nFlags - flags (not used)
/*				point - position where the button was pressed
/*
/*  Returns: None
/*
/*  Description: 
/*		Captures the click of the left mouse button. This is
/*		used for determining the posline or the start of the 
/*		selection.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnLButtonDown(UINT nFlags, CPoint point) {
	if (working) return;
	//start the selection if file open
	if (fileopen) {
		if ((point.x >= tophalf.left) && (point.x <= tophalf.right) && (point.y > tophalf.top) && (point.y < bottomhalf.bottom) && (!lbuttondown)) {
			DrawSelection(startselection, endselection);
			RestoreLine();
			linepos = (float)(point.x - LEFT) / (float)((tophalf.right - tophalf.left ));
			SaveLine();
			DrawLine();
			lbuttondown = true;
			startselection = (float)(point.x - LEFT) / (float)((tophalf.right - tophalf.left ));
			endselection = (float)(point.x - LEFT) / (float)((tophalf.right - tophalf.left ));
			DrawSelection(startselection, endselection);
		}
	}
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: nFlags - flags (not used)
/*				point - position where the button was released
/*
/*  Returns: None
/*
/*  Description:
/*		Captures when the left mouse button is released and 
/*		calls the function to calculate the coresponding file
/*		positions.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnLButtonUp(UINT nFlags, CPoint point) {
	if (fileopen) {
		//end selection - find file positions
		if (lbuttondown) {
			CalculateFilePositions();
		}
	}
	lbuttondown = false;
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: nFlags - flags (not used)
/*				point - the position of the mouse
/*
/*  Returns: None
/*
/*  Description:
/*		Captures the movement of the mouse.  This is necessary
/*		for the selection of the file in the window.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnMouseMove(UINT nFlags, CPoint point) {
	if (working) return;

	if (fileopen) {
		//calculate new selection and draws it - only draws from 
		//the end of the last part drawn to where the mouse is now
		//which prevents flickering
		if (lbuttondown) {
			RestoreLine();
			linepos = -1;
			if (point.x >= tophalf.right) {
				DrawSelection(endselection, 1);
				endselection = 1;
			} else if (point.x <= tophalf.left) {
				DrawSelection(endselection, 0);
				endselection = 0;
			} else {

			DrawSelection(endselection, ((float)(point.x - LEFT) / (float)((tophalf.right - tophalf.left ))));
			endselection = (float)((float)(point.x - LEFT)/ (float)((tophalf.right - tophalf.left )));
			}
			if (startselection == endselection) {
				linepos = (float)(point.x - LEFT) / (float)((tophalf.right - tophalf.left));
				SaveLine();
				DrawLine();
			}
		}
	}
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: nFlags - flags (not used)
/*				point - where the mouse was doubleclicked
/*
/*  Returns: None
/*
/*  Description:
/*		Captures the doubleclick of the left mouse button. 
/*		A double click means a select all.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnLButtonDblClk(UINT nFlags, CPoint point) {
	if (working) return;
	if (fileopen) {
		//select all if within our wave data
		if ((point.y > tophalf.top) && (point.y < bottomhalf.bottom) && (!lbuttondown)) {
			RestoreLine();
			linepos = -1;
			DrawSelection(startselection, endselection);
			startselection = 0;
			endselection = 1;
			DrawSelection(startselection, endselection);
			CalculateFilePositions();
		}
	}
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description: 
/*		This is called when the gotostart button is pressed.  
/*		This repositions the playback and posline to the 
/*		beginning of the file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnGotoStart() {
	MCI_SEEK_PARMS seekpar;

	if ((!fileopen) || (working) || (paused)) {
		return;
	}
	
	//remove the selection or the line
	DrawSelection(startselection, endselection);
	RestoreLine();
	startselection = 0;
	endselection = 0;
	
	linepos = 0;
	DrawSelection(startselection, endselection);
	SaveLine();
	DrawLine();
	//find new file positions
	CalculateFilePositions();

	DrawVerticalLine(lastmark);
	lastmark = -1;
	if (playing) {
		//set playback to the beginning of the file
		seekpar.dwCallback = 0;
		seekpar.dwTo = 0;
		mciSendCommand(wDeviceID, MCI_PAUSE, 0, NULL);
		mciSendCommand(wDeviceID, MCI_SEEK, MCI_NOTIFY | MCI_TO, (DWORD)(LPMCI_SEEK_PARMS) &seekpar);
		mciSendCommand(wDeviceID, MCI_PLAY, 0, NULL);
	}
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description: 
/*		This is called when the goto end button is pressed.  
/*		This repositions the playback and posline to the 
/*		end of the file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnGotoEnd() {
	MCI_SEEK_PARMS seekpar;
	if ((!fileopen) || (working) || (paused)) {
		return;
	}
	
	//remove selection or line
	DrawSelection(startselection, endselection);
	RestoreLine();
	linepos = 1;
	startselection = 1;
	endselection = 1;
	DrawSelection(startselection, endselection);
	SaveLine();
	DrawLine();
	DrawVerticalLine(lastmark);
	lastmark = -1;

	if (playing) {
		//set playback to the end of the file
		seekpar.dwCallback = 0;
		seekpar.dwTo = mci_wavelength;
		mciSendCommand(wDeviceID, MCI_PAUSE, 0, NULL);
		mciSendCommand(wDeviceID, MCI_SEEK, MCI_NOTIFY | MCI_TO, (DWORD)(LPMCI_SEEK_PARMS) &seekpar);
		mciSendCommand(wDeviceID, MCI_PLAY, 0, NULL);
	}
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		Selects from the current position in the file to the 
/*		end of the file.  
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnSelToEnd() {
	RestoreLine();
	linepos = -1;
	//erase current selection
	DrawSelection(startselection, endselection);
	endselection = 1;
	if (startselection == -1) {
		startselection = 0;
	}
	//draw new selection
	DrawSelection(startselection, endselection);
	CalculateFilePositions();
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		Selects from the current position in the fiel to the
/*		beginning of the file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnSelToStart() {
	RestoreLine();
	linepos = -1;
	//erase current selection
	DrawSelection(startselection, endselection);
	startselection = 0;
	//draw new selection
	DrawSelection(startselection, endselection);
	CalculateFilePositions();
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		Sets the selection of the file to nothing and moves the 
/*		current position of the file to the beginning of the 
/*		file. 
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnSelNone() {
	RestoreLine();
	linepos = -1;
	//remove selection
	DrawSelection(startselection, endselection);
	startselection = -1;
	endselection = -1;
	DrawSelection(startselection, endselection);
	//calculate new file positions
	CalculateFilePositions();
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		Selects the whole file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnSelAll() {
	if (working) {
		return;
	}
	if (fileopen) {
		RestoreLine();
		linepos = -1;
		//remove old selection
		DrawSelection(startselection, endselection);
		startselection = 0;
		endselection = 1;
		//draw new selection
		DrawSelection(startselection, endselection);
		CalculateFilePositions();
	}
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: xpos - position to draw the verticle line
/*
/*  Returns: None
/*
/*  Description:
/*		Inverts a line on the screen to indicate where the 
/*		current position of playback is located
/*
/*************************************************************/
/*************************************************************/
void CMainWindow::DrawVerticalLine(int xpos) {
	CRect r;
	CClientDC dc(this);
	if (xpos == linepos) {
		return;
	} 
	//calculate rect for the line
	r.left = xpos;
	r.right = xpos + 1;
	r.top = tophalf.top;
	r.bottom = bottomhalf.bottom;
	//invert rect (draw line)
	dc.InvertRect(r);
	DeleteDC(dc);
	return;
}

//////////////////////////////////////////////////////////////////////////////
////////////// END OF SELECTION FUNCTIONS ////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////




/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description: 
/*		Shows the status of the program on the screen in the
/*		designated area.
/*
/*************************************************************/
/*************************************************************/
void CMainWindow::ShowStatus() {
	CClientDC dc(this);
	CString outtext;
	CFont afont, *oldfont;
	COLORREF oldcolor, oldfontcolor;
	afont.CreatePointFont(240, "Arial");
	oldfont = dc.SelectObject(&afont);
	oldcolor = dc.SetBkColor(GrayColor);
	oldfontcolor = dc.SetTextColor(DARKBLUE);

	//only show the status if it's different than the last one
	if (laststatustext != statustext) {
		dc.TextOut(tophalf.right / 2 - 110, statusbox.top, "                                         ");
		dc.TextOut(tophalf.right / 2 - 110, statusbox.top, statustext);
		laststatustext = statustext;
	} 
	if (laststatusstate != statusstate) {
		dc.TextOut(tophalf.left + 10, statusbox.top, "                                     ");
		dc.TextOut(tophalf.left + 10, statusbox.top, statusstate);
		laststatusstate = statusstate;
	}
	dc.SelectObject(oldfont);
	dc.SetTextColor(oldfontcolor);
	dc.SetBkColor(oldcolor);
	afont.DeleteObject();

	DeleteDC(dc);
	return;
}



/*************************************************************/
/*************************************************************/
/*
/*  Parameters: nIDEvent - which timer is being called
/*
/*  Returns: None
/*
/*  Description: 
/*		Gets called whenever the timer in the program is 
/*		triggered.  This is used to update the playback time
/*		and to update the status of the program.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnTimer(UINT nIDEvent) {
	CClientDC dc(this);
	double div;
	int hours = 0;
	int minutes = 0;
	int seconds = 0;
	int milliseconds = 0;
	char chours[2], cminutes[2], cseconds[2], cmilliseconds[3];
	CString postime = "";

	MCI_STATUS_PARMS StatusParms;
	DWORD position, tmppos;
	int newpos = 0;
	if (playing) {
		StatusParms.dwCallback = 0;
		//get the current playback position of the file
		StatusParms.dwItem = MCI_STATUS_POSITION;
		mciSendCommand(wDeviceID, MCI_STATUS, MCI_STATUS_ITEM | MCI_NOTIFY, (DWORD)(LPMCI_STATUS_PARMS) &StatusParms);
		position = StatusParms.dwReturn;
		if (position >= PlayEnd){
			OnStop();
			if (loop) {
				OnPlay();
			}
			return;
		}
		//check if the position is not at the end of the file
		if (position < mci_wavelength) {
			//find new line position
			div = (float)mci_wavelength / (float)((tophalf.right - tophalf.left));
			newpos = (int)((float)position / (float)div) + tophalf.left;
			DrawVerticalLine(lastmark);
			DrawVerticalLine((int)newpos);
			lastmark = (unsigned long)newpos;
		} else {
			if (loop) {
				OnStop();
				OnPlay();
			} else {
				OnStop();
			}
		}

		//calculate time to display
		tmppos = position;
		hours = tmppos / 3600000;
		itoa(hours, chours, 10);
		tmppos -= hours * 3600000;
		minutes = tmppos / 60000;
		itoa(minutes, cminutes, 10);
		tmppos -= minutes * 60000;
		seconds = tmppos / 1000;
		itoa(seconds, cseconds, 10);
		tmppos -= seconds * 1000;
		milliseconds = (int)tmppos / 100;

		itoa(milliseconds, cmilliseconds, 10);
		
		if (hours < 10) {
			postime += "0";
		}
		postime += chours;
		postime += ":";
		if (minutes < 10) {
			postime += "0";
		}
		postime += cminutes;
		postime += ":";
		if (seconds < 10) {
			postime += "0";
		}
		postime += cseconds;
		postime += ":";
		postime += cmilliseconds;
		statustext = postime; 
	} else {
		statustext = "";
	}
	ShowStatus();
	DeleteDC(dc);
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		Destroys the window from memory.
/*
/*************************************************************/
/*************************************************************/
void CMainWindow::PostNcDestroy ()
{	
    delete this;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		Draws the layout of the window before the file info
/*		is drawn onto the window. 
/*
/*************************************************************/
/*************************************************************/
void CMainWindow::DrawLayout() {
	CClientDC dc(this);
	CRect drect, rrect;
	int pos = 0, range = 0, d = 0;
	float diff = 0;
	GetClientRect(&rrect);

	rrect.top = winrect.top;
	rrect.bottom = winrect.bottom - CONTROLBARTOP;
	rrect.right = winrect.right;
	rrect.left = winrect.left;
	
	//resize the toolbar
	ToolBar.AutoSize();
	dc.FillSolidRect(rrect, GrayColor);

	laststatustext = "";
	laststatusstate = "";

	if (fileopen) {
		//draw the rectangles
		CPen apen, *oldpen;
		if (WaveFormat.nChannels == 2) {
			dc.FillSolidRect(tophalf, BLACK);
			dc.FillSolidRect(bottomhalf, BLACK);

			apen.CreatePen(PS_SOLID, 1, RED);
			oldpen = dc.SelectObject(&apen);

			dc.MoveTo(tophalf.left, qlinetop);
			dc.LineTo(tophalf.right, qlinetop);
			dc.MoveTo(tophalf.left, qlinebottom);
			dc.LineTo(tophalf.right, qlinebottom);
			
			dc.SelectObject(oldpen);
			apen.DeleteObject();
			apen.CreatePen(PS_SOLID, 2, GrayColor);
			oldpen = dc.SelectObject(&apen);
			dc.MoveTo(tophalf.left, midline);
			dc.LineTo(tophalf.right, midline);
			dc.SelectObject(oldpen);
			apen.DeleteObject();

		} else if (WaveFormat.nChannels == 1) {
			drect.top = tophalf.top;
			drect.bottom = bottomhalf.bottom;
			drect.left = tophalf.left;
			drect.right = bottomhalf.right;
			dc.FillSolidRect(drect, BLACK);

			apen.DeleteObject();
			apen.CreatePen(PS_SOLID, 1, RED);
			oldpen = dc.SelectObject(&apen);
			dc.MoveTo(tophalf.left, midline + 2);
			dc.LineTo(tophalf.right, midline + 2);
			dc.SelectObject(oldpen);
			apen.DeleteObject();
		}

		COLORREF ref;
		CFont af;
		af.CreatePointFont(100, "Arial");
		CFont * oldfont = dc.SelectObject(&af);
		ref = dc.SetBkColor(GrayColor);
		LPTSTR st = wavefile.soundfilename.GetBuffer(255);
		dc.TextOut(1,1, (LPCTSTR)wavefile.soundfilename);
		
		dc.SelectObject(oldfont);
		dc.SetBkColor(ref);

		if ((wavefile.wd.rightchannel != NULL) || (wavefile.wd.leftchannel != NULL)){
			DrawWaveData();
		}
		DrawSelection(startselection, endselection);
	} else {
		//don't draw rectangles - draw blank window
		dc.FillSolidRect(tophalf.left, tophalf.top, bottomhalf.right, (bottomhalf.bottom - tophalf.top), DARKGRAY);
	}	
	DeleteDC(dc);

	return;
}



/*************************************************************/
/*************************************************************/
/*
/*  Parameters: nType (not used)
/*				cx (not used)
/*				cy (not used)
/*
/*  Returns: None
/*
/*  Description:
/*		This is called when the window is resized.  This is 
/*		needed in order to resize the layout of the window.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnSize(UINT nType, int cx, int cy) {
	GetClientRect(&winrect);
	//recalculate all the rectangles for the window
	toprect.top = winrect.top;
	toprect.bottom = winrect.top + TOP;
	toprect.left = winrect.left + LEFT;
	toprect.right = winrect.right - RIGHT;

	tophalf.top = winrect.top + TOP + 1;
	tophalf.bottom = (((winrect.bottom - BOTTOM) - (winrect.top + TOP)) / 2) + TOP - 2;
	tophalf.left = winrect.left + LEFT;
	tophalf.right = winrect.right - RIGHT;

	midline = tophalf.bottom + 1;

	bottomhalf.top = (((winrect.bottom - BOTTOM) - (winrect.top + TOP)) / 2) + TOP + 2;
	bottomhalf.bottom = winrect.bottom - BOTTOM;
	bottomhalf.left = winrect.left + LEFT;
	bottomhalf.right = winrect.right - RIGHT;

	statusbox.top = bottomhalf.bottom;
	statusbox.left = winrect.left + LEFT;
	statusbox.right = winrect.right - RIGHT;
	statusbox.bottom = winrect.bottom - CONTROLBARTOP;
	
	qlinetop = ((tophalf.bottom - tophalf.top) / 2) + TOP; 
	qlinebottom = ((bottomhalf.bottom - bottomhalf.top) / 2) + tophalf.bottom + 4;

	DeletePosLineList();
	posline = NULL;
	DrawLayout();
	SaveLine();
	DrawLine();
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: pOldWnd (not used)
/*
/*  Returns: None
/*
/*  Description:
/*		This is called when the focus is set back on the 
/*		program's window.  This must be called in order to 
/*		redraw the layout.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnSetFocus(CWnd *pOldWnd) {
	//redraw everything on the window
	DrawLayout();
	DrawLine();
	return;
}




///////////////////////////////////////////////////////////////////////
/////wave file stuff //////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function draws the wave data onto the window.
/*
/*************************************************************/
/*************************************************************/
void CMainWindow::DrawWaveData() {
	working = true;
	MyWaveDataNode *x;
	
	CClientDC dc(this);
	int  i, rangey, pos, apoint;
	long tempsample = 0;
	int skipp;
	float skipp2;
	float skippremaindercounter, skippremainder, diff;

	laststatustext = "";
	laststatusstate = "";
	ShowStatus();
	
	skipp2 = 0;
	skipp = 0;
	skippremainder = 0;
	skippremaindercounter = 0;
	pos = tophalf.left;
	i = 0;
	CPen pn, *oldpen;

	pn.CreatePen(PS_SOLID, 0, WAVECOLOR);
	oldpen = dc.SelectObject(&pn);

	if (tophalf.right <= 0) {
		return;
	}

	if (WaveFormat.nChannels == 2) {
		rangey = ((tophalf.bottom - tophalf.top - 2));
	} else {
		rangey = bottomhalf.bottom - tophalf.top - 4;
	}

	//calculate sizing values (how many samples to put at each x value)
	skipp2 = (float)wavefile.numsamples / (float)((tophalf.right - tophalf.left));;
	skippremainder = (float)(skipp2 - (int)skipp2);
	skipp = (int)skipp2;

	
	if (rangey <= 0) {
		return;
	} else {
		diff = (float)wavefile.max / (float)rangey;
	}

	if (wavefile.wd.rightchannel != NULL) {
		if (WaveFormat.nChannels == 2) {
			dc.MoveTo(pos, qlinetop + 1);
		} else {
			dc.MoveTo(pos, midline + 2);
		} 
		x = wavefile.wd.rightchannel;
		while (x != NULL) {
			if (skippremaindercounter >= 1) {
				i--;
				skippremaindercounter -= 1;
			} 
			if (i >= skipp) {
				i = 0;
				pos++;
				skippremaindercounter += skippremainder;
			} 

			if (diff == 0) return; // avoid div by 0
			//size the wave height to fit into window
			if (WaveFormat.nChannels == 2) {
				apoint = (int)((x->sample - (wavefile.max / 2))/diff*-1) + qlinetop - TOP - 4;
			} else {
				apoint = (int)((x->sample - (wavefile.max / 2))/diff*-1) + midline - TOP - 4;
			}
			if (WaveFormat.nChannels == 2) {
				dc.LineTo(pos, apoint + TOP + 4);
				dc.MoveTo(pos, apoint + TOP + 4);
			} else {
				dc.LineTo(pos, apoint + TOP + 7);
				dc.MoveTo(pos, apoint + TOP + 7);
			}
			i++;
			x = x->link;
		}
	}
	skippremaindercounter = 0;
	if (wavefile.wd.leftchannel != NULL) {
		i = 1;
		pos = tophalf.left;
		dc.MoveTo(pos, qlinebottom + 1);
		x = wavefile.wd.leftchannel;
		
		while (x != NULL) {
			if (skippremaindercounter >= 1) {
				i--;
				skippremaindercounter -= 1;
			} 
			if (i >= skipp) {
				i = 0;
				pos++;
				skippremaindercounter += skippremainder;
			} 
			
			if (diff == 0) return; //avoid div by 0

			if (WaveFormat.nChannels == 2) {
				apoint = (int)((x->sample - (wavefile.max / 2))/(float)diff*-1) + qlinebottom - TOP - 4;
			} else {
				apoint = (int)((x->sample - (wavefile.max / 2))/(float)diff*-1) + midline - TOP - 4;
			}
			dc.LineTo(pos, apoint + TOP + 4);
			dc.MoveTo(pos, apoint + TOP + 4);
			i++;			
			x = x->link;
		}
	}
	dc.SelectObject(oldpen);
	DeleteDC(dc);
	working = false;
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function loads the wave data from the file into 
/*		the wave data linked list.  There are a set number of 
/*		sound samples loaded into the list in order to save 
/*		memory and increase program efficency.
/*
/*************************************************************/
/*************************************************************/
void CMainWindow::LoadWaveData() {
	working = true;
	statusstate = "Scanning File...";

	DeleteWaveDataLists();
	DeletePosLineList();

	MyWaveDataNode *x, *r, *l;
	int skipit;
	int avg = 0;
	WORD dat;
	DWORD bytesread = 0;
	DWORD pd = 0;
	DWORD ps = 0;

	//calculate how many samples to skip between storing samples
	//keeps # of samples read at a fixed number
	skipit = (wavefile.datasize / WaveFormat.nBlockAlign) / 20000 * WaveFormat.nBlockAlign;


	if (wavefile.datasize > 0) {
		if (WaveFormat.nChannels == 1) {
			x = new MyWaveDataNode;
			if (mmioRead(HMmio, (char *) &dat, wavefile.bps) > 0) {
				if (wavefile.bps == 1) {
					x->sample = (WORD)(BYTE)dat;
				} else {
					x->sample = (WORD)((WORD)(dat - (wavefile.max / 2)));
				}
				
				wavefile.numsamples++;
				x->link = NULL;
				wavefile.wd.rightchannel = x;
				bytesread += wavefile.bps;
				if (bytesread >= wavefile.datasize) {
					goto endit;
				}
				r = x;
			} else { 
				goto endit;
			}
			while (1) {
				pd = mmioSeek(HMmio, skipit, SEEK_CUR);
				if (((pd - 44) % (WaveFormat.nBlockAlign / WaveFormat.nChannels)) != 0) {
					MessageBox("oops - miss-aligned position on the file!");
					goto endit;
				}
				x = new MyWaveDataNode;
				if (mmioRead(HMmio, (char *) &dat, wavefile.bps) > 0) {
					wavefile.numsamples++;
					if (wavefile.bps == 1) {
						x->sample = (WORD)(BYTE)dat;
					} else {				
						x->sample = (WORD)(dat - (wavefile.max / 2));
					}
					x->link = NULL;
					r->link = x;
					r = x;
					bytesread += wavefile.bps;
					if (bytesread >= wavefile.datasize) {
						goto endit;
					}
				} else {
					break;
				}
			}
		} else if (WaveFormat.nChannels == 2) {
			x = new MyWaveDataNode;
			if (mmioRead(HMmio, (char *) &dat, wavefile.bps) > 0) {
				if (wavefile.bps == 1) {
					x->sample = (WORD)(BYTE)dat;
				} else {
					x->sample = (WORD)(dat - (wavefile.max / 2));
				}
				wavefile.numsamples++;
				x->link = NULL;
				wavefile.wd.rightchannel = x;
				r = x;

				bytesread += wavefile.bps;
				if (bytesread >= wavefile.datasize) {
					goto endit;
				}

				x = new MyWaveDataNode;
				if (mmioRead(HMmio, (char *) &dat, wavefile.bps) > 0) {
					if (wavefile.bps == 1) {
						x->sample = (WORD)(BYTE)dat;
					} else {
						x->sample = (WORD)(dat - (wavefile.max / 2));
					}
					x->link = NULL;
					wavefile.wd.leftchannel = x;
					l = x;
					bytesread += wavefile.bps;
					if (bytesread >= wavefile.datasize) {
						goto endit;
					}
				} else {
					goto endit;
				}
			} else { 
				goto endit;
			}
			while (1) {
				pd = mmioSeek(HMmio, skipit, SEEK_CUR);
				x = new MyWaveDataNode;
				if (mmioRead(HMmio, (char *) &dat, wavefile.bps) > 0) {
					if (wavefile.bps == 1) {
						x->sample = (WORD)(BYTE)dat;
					} else {
						x->sample = (WORD)(dat - (wavefile.max / 2));
					}
					wavefile.numsamples++;
					x->link = NULL;
					r->link = x;
					r = x;

					bytesread += wavefile.bps;
					if (bytesread >= wavefile.datasize) {
						goto endit;
					}
					
					x = new MyWaveDataNode;
					if (mmioRead(HMmio, (char *) &dat, wavefile.bps) > 0) {
						if (wavefile.bps == 1) {
							x->sample = (WORD)(BYTE)dat;
						} else {
							x->sample = (WORD)(dat - (wavefile.max / 2));
						}
						x->link = NULL;
						l->link = x;
						l = x;
						bytesread += wavefile.bps;
						if (bytesread >= wavefile.datasize) {
							goto endit;
						}
					} else {
						break;
					}
				} else {
					break;
				}
			}
		} else {
			MessageBox("File Type is Not Supported!");
		}
	}
endit:
	statusstate = "";
	statustext = "";
	working = false;
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: 0 if function succeeds.  
/*
/*  Description:
/*		This function reads the header of the wave file.
/*
/*************************************************************/
/*************************************************************/
int CMainWindow::GetWaveInfo() {
	MMCKINFO	mmckinfoParent;
	laststatusstate = "";		
	statusstate = "Reading Wave Info...";
	ShowStatus();

	//use these functions to go through and read the data from the wave file header
	if ((HMmio = mmioOpen((char *)(LPCTSTR)wavefile.modfile, 0, MMIO_READ))){
		mmckinfoParent.fccType = mmioFOURCC('W', 'A', 'V', 'E'); 
		if (mmioDescend(HMmio, (LPMMCKINFO)&mmckinfoParent, 0, MMIO_FINDRIFF)){
				MessageBox("Error! This file doesn't contain a WAVE!");
out:			mmioClose(HMmio, 0);

			goto out2;
		} 
		statustext = "25%";
		ShowStatus();
 		MmckinfoSubchunk.ckid = mmioFOURCC('f', 'm', 't', ' '); 
		if (mmioDescend(HMmio, &MmckinfoSubchunk, &mmckinfoParent, MMIO_FINDCHUNK)){
			MessageBox("Error! Required fmt chunk was not found!");
			goto out;
		}

		statustext = "50%";
		ShowStatus();

		if (mmioRead(HMmio, (HPSTR)&WaveFormat, MmckinfoSubchunk.cksize) != (LRESULT)MmckinfoSubchunk.cksize){
			MessageBox("Error reading the fmt chunk!");
			goto out;
		}

		statustext = "75%";
		ShowStatus();
		mmioAscend(HMmio, &MmckinfoSubchunk, 0); 

		MmckinfoSubchunk.ckid = mmioFOURCC('d', 'a', 't', 'a'); 
		if (mmioDescend(HMmio, &MmckinfoSubchunk, &mmckinfoParent, MMIO_FINDCHUNK)){
			MessageBox("Error reading the data chunk!");
			goto out;
		}
		wavefile.datasize = WaveDataPlayed = MmckinfoSubchunk.cksize;
		
		CurrentFileDataPos = FileDataStartPos = mmioSeek(HMmio, 0, SEEK_CUR);
		
		if ((WaveFormat.nBlockAlign == 1) || ((WaveFormat.nBlockAlign == 2) && (WaveFormat.nChannels == 2))){
			wavefile.max = 255;
			wavefile.bps = 1;
			zeroval = 128;
		} else if (((WaveFormat.nBlockAlign == 2) && (WaveFormat.nChannels == 1)) || ((WaveFormat.nBlockAlign == 4) && (WaveFormat.nChannels == 2))) {
			wavefile.max = 65535;
			wavefile.bps = 2;
			zeroval = 0;
		}

		statustext = "100%";
		ShowStatus();
		
		LoadWaveData();
		DrawLayout();

		return(0);
	}

	MessageBox("Can't find the WAVE file!");

out2:
		DrawLayout();
		return(-2);
	
	return(-1);

}



/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		Opens the wave file after being modified.  This is
/*		done without any interaction from the user.
/*
/*************************************************************/
/*************************************************************/
void CMainWindow::OpenFile() {
	nextmark = 0;
	lastmark = 0;

	wavefile.wd.rightchannel = NULL;
	wavefile.wd.leftchannel = NULL;

	wavefile.bps = 0;
	wavefile.max = 0;
	
	lbuttondown = false;
	nextmark = -1;
	lastmark = -1;
	wavefile.numsamples = 0;
	fileopen = true;
	CreateMyThread("getwaveinfo");
	DrawSelection(startselection, endselection);
	DrawLine();
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function determines all the temp files needed for
/*		the program.
/*
/*************************************************************/
/*************************************************************/
void CMainWindow::FindFolders() {
	CString prevstring = "";
	CString FolderString = "";

	//extract the folder name from the soundfilepath
	for (int ds = 0; ds < wavefile.soundfilepath.GetLength(); ds++) {
		if (wavefile.soundfilepath[ds] == '\\') {
			prevstring += wavefile.soundfilepath[ds];
			FolderString += prevstring;
			prevstring = "";
		} else {
			prevstring += wavefile.soundfilepath[ds];
		}
	}

	//build file names	
	wavefile.modfile = wavefile.soundfilepath + "_modfile";
	wavefile.workfile = wavefile.soundfilepath + "_workfile";
	wavefile.undofile = wavefile.soundfilepath + "_undo";
	wavefile.tmpfile = wavefile.soundfilepath + "_tmp";
	wavefile.SoundFileFolder = FolderString;
	return;
}



/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function displays the open dialog box and allows
/*		the user to select a file.  Once a file is selected, 
/*		it is then opened for editing/playback.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnOpen(){
	DrawLayout();
	DrawLine();

	//pop up default windows open file dialog
	CFileDialog  * dlg = new CFileDialog (true, NULL, NULL, OFN_EXPLORER | OFN_NONETWORKBUTTON | OFN_HIDEREADONLY, "WAVE Files (*.wav)|*.wav||", this);

	if(dlg->DoModal()== IDOK) {
		if (fileopen) {
			if (!OnCloseFile()) return;
		}
		if (wavefile.soundfilepath != dlg->GetPathName()) {
			wavefile.soundfilepath = dlg->GetPathName();
			wavefile.soundfilename = dlg->GetFileName();
			
			FindFolders();

			nextmark = 0;
			lastmark = 0;

			wavefile.wd.rightchannel = NULL;
			wavefile.wd.leftchannel = NULL;

			wavefile.bps = 0;
			wavefile.max = 0;
			
			lbuttondown = false;
			startselection = -1;
			endselection = -1;
			nextmark = -1;
			lastmark = -1;
			wavefile.numsamples = 0;
			fileopen = true;
			DeleteFile(wavefile.modfile);
			DeleteFile(wavefile.undofile);
			DeleteFile(wavefile.clipfile);
			DeleteFile(wavefile.tmpfile);
			CString this1;
			this1 = dlg->GetPathName();
			CopyFile(this1, wavefile.modfile, false);
			ext = "Waveform";
			CreateMyThread("getwaveinfo");

			///////enable and disable buttons///////
			ToolBar.EnableButton(IDC_PLAY,TRUE);
			ToolBar.EnableButton(IDC_REWIND,TRUE);
			ToolBar.EnableButton(IDC_FASTFWD,TRUE);
			ToolBar.EnableButton(IDC_STOP,FALSE);
			ToolBar.EnableButton(IDC_GOTOEND,TRUE);
			ToolBar.EnableButton(IDC_GOTOSTART,TRUE);
			////////////////////////////////////////
		}
	} else {
		//open was cancelled 
		DrawLayout();
		DrawSelection(startselection, endselection);
		DrawLine();
	}
	return;	
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function saves the changes made to the file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnSave(){
	if ((!fileopen) || (playing) || (paused) || (working)) return;
	mw->statusstate = "Saving...";
	if (!DeleteFile(wavefile.soundfilepath)) {
		MessageBox("Save Not Successful");
		return;
	}
	if (!CopyFile(wavefile.modfile, wavefile.soundfilepath, false)) {
		MessageBox("Save Not Successful");
		return;
	}
	changedsincesaved = false;
	mw->statusstate = "";

	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function displays the save as... dialog box and 
/*		allows the user to name the location and name of the 
/*		new file that the current open file should be saved
/*		to.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnSaveAs() {
	if ((!fileopen) || (playing) || (paused) || (working)) return;
	mw->statusstate = "Save As...";

	//use default windows save as dialog
	CFileDialog  * dlg = new CFileDialog (false, NULL, NULL, OFN_EXPLORER | OFN_NONETWORKBUTTON | OFN_HIDEREADONLY, "WAVE Files (*.wav)|*.wav||", this);

saveit:
	if(dlg->DoModal()== IDOK) {
		CString d = dlg->GetPathName();
		if (!CopyFile(wavefile.modfile, d, true)) {
			//if exists
			if (MessageBox("File Already Exists!  Do you want to replace it?", "File Exists", MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2 | MB_APPLMODAL) == IDYES) {
				wavefile.soundfilepath = dlg->GetPathName();
				wavefile.soundfilename = dlg->GetFileName();
				changedsincesaved = false;
				DeleteFile(d);
				CopyFile(wavefile.modfile, d, false);
				FindFolders();
				CopyFile(wavefile.soundfilepath, wavefile.modfile, false);
			} else {
				goto saveit;
			}
		} else {
			wavefile.soundfilepath = dlg->GetPathName();
			wavefile.soundfilename = dlg->GetFileName();
			changedsincesaved = false;
			FindFolders();
			CopyFile(wavefile.soundfilepath, wavefile.modfile, false);
		}
	}
	
	DrawLayout();
	DrawLine();
	mw->statusstate = "";
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function is used to close the file after it has
/*		been modified. This is always followed by a re-opening
/*		of the file.  This is done without the user knowing
/*		about it. 
/*
/*************************************************************/
/*************************************************************/
void CMainWindow::FakeClose() {
	if (fileopen) {
		DeleteWaveDataLists();
		wavefile.wd.rightchannel = NULL;
		wavefile.wd.leftchannel = NULL;
		DeletePosLineList();
		wavefile.bps = 0;
		wavefile.max = 0;
		fileopen = false;
		wavefile.numsamples = 0;
		lbuttondown = false;
		nextmark = -1;
		lastmark = -1;
	}
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: false if cancel is selected, else true
/*
/*  Description:
/*		This function is called when the user selects the 
/*		close file option from the file menu.  If changes have
/*		been made to the file, it asks the user if they would
/*		like to save the changes to the file.  
/*
/*************************************************************/
/*************************************************************/
afx_msg bool CMainWindow::OnCloseFile() {
	if (fileopen) {
		mw->statusstate = "Closing File...";
		if (changedsincesaved) {
			int resp = MessageBox("Do you wish to save your changes before you close the file?", "Save Changes?", MB_YESNOCANCEL | MB_ICONQUESTION | MB_DEFBUTTON1 | MB_APPLMODAL);
			if (resp == IDYES) {
				OnSave();
			} else if (resp == IDCANCEL) {
				return false;
			}
		}
		DeleteFile(wavefile.workfile);
		DeleteFile(wavefile.tmpfile);
		DeleteFile(wavefile.undofile);
		DeleteFile(wavefile.clipfile); 
		DeleteFile(wavefile.modfile);

		startselection = -1;
		endselection = -1;
		posline = NULL;
		DrawSelection(startselection, endselection);
		DrawLine();
		changedsincesaved = false;
		stopped = true;
		OnStop();
	
		DeleteWaveDataLists();
		DeletePosLineList();

		///////enable and disable buttons///////
		ToolBar.EnableButton(IDC_PLAY,FALSE);
		ToolBar.EnableButton(IDC_REWIND,FALSE);
		ToolBar.EnableButton(IDC_FASTFWD,FALSE);
		ToolBar.EnableButton(IDC_STOP,FALSE);
		ToolBar.EnableButton(IDC_GOTOEND,FALSE);
		ToolBar.EnableButton(IDC_GOTOSTART,FALSE);
		ToolBar.EnableButton(IDC_PAUSE,FALSE);
		ToolBar.EnableButton(IDC_RECORD, TRUE);
		////////////////////////////////////////

		wavefile.wd.rightchannel = NULL;
		wavefile.wd.leftchannel = NULL;

		wavefile.bps = 0;
		wavefile.max = 0;
		fileopen = false;
		wavefile.numsamples = 0;
		lbuttondown = false;
		nextmark = -1;
		lastmark = -1;
		linepos = -1;
		
		wavefile.SoundFileFolder = "";
		wavefile.soundfilename = "";
		wavefile.soundfilepath = "";

		DrawLayout();
		DrawLine();
		mw->statusstate = "";
		pw.count = 0;
		pw.freq = 0;
		pw.amp = 0;
		pw.length = 0;
	}
	return true;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function reverts the file to the last saved
/*		version of the file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnRevert() {
	if ((!playing) && (!working)) {
		working = true;
		mw->statusstate = "Reverting...";
		FakeClose();
		DeleteFile(wavefile.tmpfile);
		DeleteFile(wavefile.workfile);
		DeleteFile(wavefile.modfile);
		DeleteFile(wavefile.undofile);
		startselection = -1;
		endselection = -1;
		changedsincesaved = false;
		linepos = -1;
		CopyFile(wavefile.soundfilepath, wavefile.modfile, false);
		OpenFile();
		mw->statusstate = "";
		working = false;

	}
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function saves the undo data before the file is
/*		modified.  This is needed in order to later undo
/*		the changes made to the file. 
/*
/*************************************************************/
/*************************************************************/
void SaveUndoData() {
	mw->statusstate = "Saving Undo Data...";
	DeleteFile(mw->wavefile.undofile);
	CopyFile(mw->wavefile.modfile, mw->wavefile.undofile, false);
	mw->statusstate = "";
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description: 
/*		This function is used to copy the data to the clipboard
/*		file.  
/*
/*************************************************************/
/*************************************************************/
void SaveData() {
	if (!mw->fileopen) return;
	mw->statusstate = "Copying to Clipboard...";
	fstream inf, outf, f;

	unsigned int buffersize = 10000;
	char *databuffer = (char *)malloc(buffersize);

	DWORD bytesread = 0;
	DWORD totalbytesread = 0;
	DWORD bytestoread = 0;

	bytestoread = mw->FileSelEnd - mw->FileSelStart;
	
	mw->SizeOfClipboardData = bytestoread;
	if (bytestoread > 0) {
		DeleteFile(mw->wavefile.workfile);
		outf.open(mw->wavefile.workfile, ios::out | ios::binary);
		inf.open(mw->wavefile.modfile, ios::in | ios::binary);
		inf.seekg(mw->FileSelStart, ios::beg);
		while (1) {
			if (bytestoread < buffersize) {	
				inf.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				outf.write((char *) databuffer, buffersize);
			}		
			bytestoread -= buffersize;
		}
	}
	
	free(databuffer);

	outf.close();
	inf.close();
	mw->statusstate = "";
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function cuts the selected portion of the wave
/*		file from the file and saves it to the clipboards.
/*
/*************************************************************/
/*************************************************************/
void Cut() {
	if (!mw->fileopen) return;
	mw->statusstate = "Cutting Data...";
	fstream inf, outf;
	
	DeleteFile(mw->wavefile.tmpfile);
	inf.open(mw->wavefile.modfile, ios::in | ios::binary);
	outf.open(mw->wavefile.tmpfile, ios::out | ios::binary);
	
	unsigned int bufferisze = 10000;
	char *databuffer = (char *)malloc(bufferisze);

	DWORD i = 0;
	DWORD bytesread = 0;
	DWORD bytestoread = mw->FileSelStart - mw->FileDataStartPos;


	DWORD dtmp = mw->wavefile.datasize - mw->SizeOfClipboardData;
	
	DWORD wfchunksize = dtmp + 36;
	inf.read((char *) databuffer, 4);
	outf.write((char *) databuffer, 4); 
	outf.write((char *) &wfchunksize, 4);

	inf.seekg(4, ios::cur);
	inf.read((char *) databuffer, 32);
	outf.write((char *) databuffer, 32); 
	outf.write((char *) &dtmp, 4);
	outf.seekg(4, ios::cur);

	if (bytestoread > 0) {
		inf.seekg(mw->FileDataStartPos, ios::beg);
		while (!inf.eof()) {
			if (bytestoread < bufferisze) {	
				inf.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, bufferisze);
				outf.write((char *) databuffer, bufferisze);
			}		
			bytestoread -= bufferisze;
		}
	}

	inf.seekg(mw->FileSelEnd, ios::beg);
	bytestoread = (mw->wavefile.datasize + mw->FileDataStartPos) - mw->FileSelEnd;
	if (bytestoread > 0) {
		while (!inf.eof()) {
			if (bytestoread < bufferisze) {
				inf.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, bufferisze);
				outf.write((char *) databuffer, bufferisze);
			} 
			bytestoread -= bufferisze;
		}
	}
	inf.close();
	outf.close();

	free(databuffer);

	DeleteFile(mw->wavefile.modfile);
	CopyFile(mw->wavefile.tmpfile, mw->wavefile.modfile, false);
	DeleteFile(mw->wavefile.tmpfile);
	mw->statusstate = "";
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function pastes the data from the clipboard file
/*		into the open file.
/*
/*************************************************************/
/*************************************************************/
void Paste() {
	if (!mw->fileopen) return;
	mw->statusstate = "Pasting Data...";
	fstream inf, inf2, outf;
	
	DeleteFile(mw->wavefile.tmpfile);
	inf.open(mw->wavefile.modfile, ios::in | ios::binary);
	inf2.open(mw->wavefile.workfile, ios::in | ios::binary);
	outf.open(mw->wavefile.tmpfile, ios::out | ios::binary);
	
	unsigned int buffersize = 10000;
	char *databuffer = (char *)malloc(buffersize);

	DWORD i = 0;
	DWORD bytesread = 0;
	DWORD bytestoread = mw->FileSelStart - mw->FileDataStartPos;


	DWORD dtmp = mw->SizeOfClipboardData + mw->wavefile.datasize - (mw->FileSelEnd - mw->FileSelStart);
	
	DWORD wfchunksize = dtmp + 36;
	inf.read((char *) databuffer, 4);
	outf.write((char *) databuffer, 4); 
	outf.write((char *) &wfchunksize, 4);

	inf.seekg(4, ios::cur);
	inf.read((char *) databuffer, 32);
	outf.write((char *) databuffer, 32); 
	outf.write((char *) &dtmp, 4);
	inf.seekg(4, ios::cur);

	if (bytestoread > 0) {
		while (!inf.eof()) {
			if (bytestoread < buffersize) {	
				inf.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				outf.write((char *) databuffer, buffersize);
			}		
			bytestoread -= buffersize;
		}
	}
	bytestoread = mw->SizeOfClipboardData;
	if (bytestoread > 0) {
		while (!inf2.eof()) {
			if (bytestoread < buffersize) {
				inf2.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf2.read((char *) databuffer, buffersize);
				outf.write((char *) databuffer, buffersize);
			} 
			bytestoread -= buffersize;
		}
	}
	
	inf.seekg(mw->FileSelEnd, ios::beg);
	bytestoread = (mw->wavefile.datasize + mw->FileDataStartPos) - mw->FileSelEnd;
	if (bytestoread > 0) {
		while (!inf.eof()) {
			if (bytestoread < buffersize) {
				inf.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				outf.write((char *) databuffer, buffersize);
			} 
			bytestoread -= buffersize;
		}
	}
	inf.close();
	inf2.close();
	outf.close();

	free(databuffer);

	DeleteFile(mw->wavefile.modfile);
	CopyFile(mw->wavefile.tmpfile, mw->wavefile.modfile, false);
	DeleteFile(mw->wavefile.tmpfile);
	mw->statusstate = "";
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function reloads the file after changes have been
/*		made to it.
/*
/*************************************************************/
/*************************************************************/
void ReloadFile() {
	if (!mw->fileopen) return;
	mw->statusstate = "Reloading...";
	mw->FakeClose();
	mw->OpenFile();
	mw->statusstate = "";
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function replaces the selected portion of the file
/*		with silence.
/*
/*************************************************************/
/*************************************************************/
void Clear() {
	if (!mw->fileopen) return;
	mw->statusstate = "Clearing Data...";
	fstream inf, outf;
	
	DeleteFile(mw->wavefile.tmpfile);
	inf.open(mw->wavefile.modfile, ios::in | ios::binary);
	outf.open(mw->wavefile.tmpfile, ios::out | ios::binary);
	
	unsigned int buffersize = 10000;
	char *databuffer = (char *)malloc(buffersize);

	DWORD i = 0;
	DWORD bytesread = 0;
	DWORD bytestoread = mw->FileSelStart;

	if (bytestoread > 0) {
		while (!inf.eof()) {
			if (bytestoread < buffersize) {	
				inf.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				outf.write((char *) databuffer, buffersize);
			}		
			bytestoread -= buffersize;
		}
	}

	int zerobytes = 0;
	int v = mw->zeroval;
	int selsize = mw->FileSelEnd - mw->FileSelStart;
	while(zerobytes < selsize) {
		outf.write((char *) &v, mw->wavefile.bps);
		zerobytes += mw->wavefile.bps;
	}

	inf.seekg(mw->FileSelEnd, ios::beg);
	bytestoread = (mw->wavefile.datasize + mw->FileDataStartPos) - mw->FileSelEnd;
	if (bytestoread > 0) {
		while (!inf.eof()) {
			if (bytestoread < buffersize) {
				inf.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				outf.write((char *) databuffer, buffersize);
			} 
			bytestoread -= buffersize;
		}
	}
	inf.close();
	outf.close();

	free(databuffer);

	DeleteFile(mw->wavefile.modfile);
	CopyFile(mw->wavefile.tmpfile, mw->wavefile.modfile, false);
	DeleteFile(mw->wavefile.tmpfile);
	mw->statusstate = "";
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function cuts everything except the selected area
/*		from the file.
/*
/*************************************************************/
/*************************************************************/
void Crop() {
	if (!mw->fileopen) return;
	mw->statusstate = "Cropping Data...";
	fstream inf, outf;
	
	DeleteFile(mw->wavefile.tmpfile);
	inf.open(mw->wavefile.modfile, ios::in | ios::binary);
	outf.open(mw->wavefile.tmpfile, ios::out | ios::binary);
	
	unsigned int buffersize = 10000;
	char *databuffer = (char *)malloc(buffersize);
	
	DWORD i = 0;
	DWORD bytesread = 0;
	DWORD bytestoread = mw->FileSelEnd - mw->FileSelStart;
	
	DWORD wfchunksize = bytestoread + 36;
	inf.read((char *) databuffer, 4);
	outf.write((char *) databuffer, 4); 
	outf.write((char *) &wfchunksize, 4);

	inf.seekg(4, ios::cur);
	inf.read((char *) databuffer, 32);
	outf.write((char *) databuffer, 32); 
	outf.write((char *) &bytestoread, 4);
	inf.seekg(4, ios::cur);
	
	if (bytestoread > 0) {
		inf.seekg(mw->FileSelStart, ios::beg);
		while (!inf.eof()) {
			if (bytestoread < buffersize) {	
				inf.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				outf.write((char *) databuffer, buffersize);
			}		
			bytestoread -= buffersize;
		}
	}

	inf.close();
	outf.close();

	free(databuffer);

	DeleteFile(mw->wavefile.modfile);
	CopyFile(mw->wavefile.tmpfile, mw->wavefile.modfile, false);
	DeleteFile(mw->wavefile.tmpfile);
	mw->statusstate = "";
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function inserts the specified length of silence
/*		into the file at the specified position. 
/*
/*************************************************************/
/*************************************************************/
void InsertSilence() {
	if (!mw->fileopen) return;
	mw->statusstate = "Inserting Silence...";
	fstream inf, inf2, outf;
	
	DeleteFile(mw->wavefile.tmpfile);
	inf.open(mw->wavefile.modfile, ios::in | ios::binary);
	inf2.open(mw->wavefile.workfile, ios::in | ios::binary);
	outf.open(mw->wavefile.tmpfile, ios::out | ios::binary);
	
	unsigned int buffersize = 10000;
	char *databuffer = (char *)malloc(buffersize);

	float todo = (float)mw->lensilence / (float)1000;

	DWORD silencebytes = (unsigned long)((float)todo * (float)WaveFormat.nAvgBytesPerSec);
	if ((silencebytes % WaveFormat.nChannels) != 0) {
		silencebytes += mw->wavefile.bps;
	}

	DWORD i = 0;
	DWORD bytesread = 0;
	DWORD bytestoread = mw->FileSelStart - mw->FileDataStartPos;

	DWORD dtmp = mw->wavefile.datasize + silencebytes - (mw->FileSelEnd - mw->FileSelStart);
	
	DWORD wfchunksize = dtmp + 36;
	inf.read((char *) databuffer, 4);
	outf.write((char *) databuffer, 4); 
	outf.write((char *) &wfchunksize, 4);

	inf.seekg(4, ios::cur);
	inf.read((char *) databuffer, 32);
	outf.write((char *) databuffer, 32); 
	outf.write((char *) &dtmp, 4);
	inf.seekg(4, ios::cur);

	if (bytestoread > 0) {
		while (!inf.eof()) {
			if (bytestoread < buffersize) {	
				inf.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				outf.write((char *) databuffer, buffersize);
			}		
			bytestoread -= buffersize;
		}
	}

	DWORD nullbits = 0;
	int v = mw->zeroval;
	if (silencebytes > 0) {
		for (nullbits = 0; nullbits < silencebytes; nullbits+=mw->wavefile.bps){
			outf.write((char *) &v, mw->wavefile.bps);
		}
	}
	
	inf.seekg(mw->FileSelEnd, ios::beg);
	bytestoread = (mw->wavefile.datasize + mw->FileDataStartPos) - mw->FileSelEnd;
	if (bytestoread > 0) {
		while (!inf.eof()) {
			if (bytestoread < buffersize) {
				inf.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				outf.write((char *) databuffer, buffersize);
			} 
			bytestoread -= buffersize;
		}
	}
	inf.close();
	inf2.close();
	outf.close();

	free(databuffer);

	DeleteFile(mw->wavefile.modfile);
	CopyFile(mw->wavefile.tmpfile, mw->wavefile.modfile, false);
	DeleteFile(mw->wavefile.tmpfile);
	mw->statusstate = "";
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/* 
/*  Description:
/*		This function copies the selected portion of the file
/*		to a new file.
/*
/*************************************************************/
/*************************************************************/
void CopyToNew() {
	if (!mw->fileopen) return;
	mw->statusstate = "Copy to New...";
	CFileDialog  * dlg = new CFileDialog (false, NULL, NULL, OFN_EXPLORER | OFN_NONETWORKBUTTON | OFN_HIDEREADONLY, "WAVE Files (*.wav)|*.wav||", mw);
	
	CString soundfilepath, soundfilename, ext;
saveit:
	if(dlg->DoModal()== IDOK) {
		CString d = dlg->GetPathName();
		if (!CopyFile(mw->wavefile.modfile, d, true)) {
			if (mw->MessageBox("File Already Exists!  Do you want to replace it?", "File Exists", MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2 | MB_APPLMODAL) == IDYES) {
				soundfilepath = dlg->GetPathName();
				soundfilename = dlg->GetFileName();
				ext = dlg->GetFileExt();
				mw->changedsincesaved = false;
			} else {
				goto saveit;
			}
		} else {
			soundfilepath = dlg->GetPathName();
			soundfilename = dlg->GetFileName();
			ext = dlg->GetFileExt();
			mw->changedsincesaved = false;
		}
	} else {
		mw->MessageBox("File Not Saved");
		mw->statusstate = "";
		return;
	}
	
	mw->statusstate = "Copying...";
	fstream inf, outf;
	
	unsigned int buffersize = 10000;
	char *databuffer = (char *)malloc(buffersize);

	DWORD bytesread = 0;
	DWORD totalbytesread = 0;
	DWORD bytestoread = 0;

	bytestoread = mw->FileSelEnd - mw->FileSelStart;


	DeleteFile(soundfilepath);
	outf.open(soundfilepath, ios::out | ios::binary);
	inf.open(mw->wavefile.modfile, ios::in | ios::binary);

	DWORD wfchunksize = bytestoread + 36;

	inf.read((char *) databuffer, 4);
	outf.write((char *) databuffer, 4); 
	outf.write((char *) &wfchunksize, 4);
	inf.seekg(4, ios::cur);

	inf.read((char *) databuffer, 32);
	outf.write((char *) databuffer, 32); 
	
	outf.write((char *) &bytestoread, 4);
	
	if (bytestoread > 0) {
		inf.seekg(mw->FileSelStart, ios::beg);
		while (1) {
			if (bytestoread < buffersize) {	
				inf.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				outf.write((char *) databuffer, buffersize);
			}		
			bytestoread -= buffersize;
		}
	}
	outf.close();
	inf.close();

	free(databuffer); // free up buffer memory

	mw->OnCloseFile();

	mw->wavefile.soundfilename = soundfilename;
	mw->ext = ext;
	mw->wavefile.soundfilepath = soundfilepath;
	mw->FindFolders();
	CopyFile(mw->wavefile.soundfilepath, mw->wavefile.modfile, false);
	mw->OpenFile();
	mw->statusstate = "";

	///////enable and disable buttons///////
	mw->ToolBar.EnableButton(IDC_PLAY,TRUE);
	mw->ToolBar.EnableButton(IDC_REWIND,TRUE);
	mw->ToolBar.EnableButton(IDC_FASTFWD,TRUE);
	mw->ToolBar.EnableButton(IDC_STOP,FALSE);
	mw->ToolBar.EnableButton(IDC_GOTOEND,TRUE);
	mw->ToolBar.EnableButton(IDC_GOTOSTART,TRUE);
	mw->ToolBar.EnableButton(IDC_PAUSE,FALSE);
	////////////////////////////////////////


	return;
}



/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function pastes the data from the clipboard to
/*		a new file.
/*
/*************************************************************/
/*************************************************************/
void PasteToNew() {
	if (!mw->fileopen) return;
	mw->statusstate = "Paste To New...";
	CFileDialog  * dlg = new CFileDialog (false, NULL, NULL, OFN_EXPLORER | OFN_NONETWORKBUTTON | OFN_HIDEREADONLY, "WAVE Files (*.wav)|*.wav||", mw);
	
	CString soundfilepath, soundfilename, ext;
saveit:
	if(dlg->DoModal()== IDOK) {
		CString d = dlg->GetPathName();
		if (!CopyFile(mw->wavefile.modfile, d, true)) {
			if (mw->MessageBox("File Already Exists!  Do you want to replace it?", "File Exists", MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2 | MB_APPLMODAL) == IDYES) {
				soundfilepath = dlg->GetPathName();
				soundfilename = dlg->GetFileName();
				ext = dlg->GetFileExt();
				mw->changedsincesaved = false;
			} else {
				goto saveit;
			}
		} else {
			soundfilepath = dlg->GetPathName();
			soundfilename = dlg->GetFileName();
			ext = dlg->GetFileExt();
			mw->changedsincesaved = false;
		}
	} else {
		mw->MessageBox("File Not Saved");
		mw->statusstate = "";
		return;
	}
	

	mw->statusstate = "Pasting...";
	fstream inf, inf2, outf;

	unsigned int buffersize = 10000;
	char *databuffer = (char *)malloc(buffersize);

	DWORD bytesread = 0;
	DWORD totalbytesread = 0;
	DWORD bytestoread = 0;

	DeleteFile(soundfilepath);

	bytestoread = mw->SizeOfClipboardData;

	inf.open(mw->wavefile.workfile, ios::in | ios::binary);
	inf2.open(mw->wavefile.modfile, ios::in | ios::binary);
	outf.open(soundfilepath, ios::out | ios::binary);

	DWORD wfchunksize = bytestoread + 36;

	inf2.read((char *) databuffer, 4);
	outf.write((char *) databuffer, 4); 
	outf.write((char *) &wfchunksize, 4);
	inf2.seekg(4, ios::cur);

	inf2.read((char *) databuffer, 32);
	outf.write((char *) databuffer, 32); 
	
	outf.write((char *) &bytestoread, 4);
	if (bytestoread > 0) {
		while (1) {
			if (bytestoread < buffersize) {	
				inf.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				outf.write((char *) databuffer, buffersize);
			}		
			bytestoread -= buffersize;
		}
	}
	outf.close();
	inf.close();
	inf2.close();

	free(databuffer);	// free up the data buffer
	
	mw->OnCloseFile();

	mw->wavefile.soundfilename = soundfilename;
	mw->ext = ext;
	mw->wavefile.soundfilepath = soundfilepath;
	mw->FindFolders();
	CopyFile(mw->wavefile.soundfilepath, mw->wavefile.modfile, false);
	mw->OpenFile();
	mw->statusstate = "";

	///////enable and disable buttons///////
	mw->ToolBar.EnableButton(IDC_PLAY,TRUE);
	mw->ToolBar.EnableButton(IDC_REWIND,TRUE);
	mw->ToolBar.EnableButton(IDC_FASTFWD,TRUE);
	mw->ToolBar.EnableButton(IDC_STOP,FALSE);
	mw->ToolBar.EnableButton(IDC_GOTOEND,TRUE);
	mw->ToolBar.EnableButton(IDC_GOTOSTART,TRUE);
	mw->ToolBar.EnableButton(IDC_PAUSE,FALSE);
	////////////////////////////////////////


	return;
}




/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function reverses the sound in the selected
/*		portion of the file.
/*
/*************************************************************/
/*************************************************************/
void Reverse() {
	if (!mw->fileopen) return;
	mw->statusstate = "Reversing Sound...";
	fstream inf, outf;
	DeleteFile(mw->wavefile.tmpfile);
	inf.open(mw->wavefile.modfile, ios::in | ios::binary);
	outf.open(mw->wavefile.tmpfile, ios::out | ios::binary);
	
	unsigned int buffersize = 10000;
	char *databuffer = (char *)malloc(buffersize);

	DWORD i = 0;
	DWORD bytesread = 0;
	DWORD bytestoread = mw->FileSelStart;

	if (bytestoread > 0) {
		while (!inf.eof()) {
			if (bytestoread < buffersize) {	
				inf.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				outf.write((char *) databuffer, buffersize);
			}		
			bytestoread -= buffersize;
		}
	}

	BYTE r, l, r2, l2;
	r = l = 0;
	bytestoread = mw->FileSelEnd - mw->FileSelStart;

	DWORD onbyte;
	DWORD y;
	if (bytestoread > 0) {
		while (!inf.eof()) {
			if (bytestoread < buffersize) {
				inf.seekg((mw->FileSelStart), ios::beg);
				inf.read((char *) databuffer, bytestoread);
				y = bytestoread - 1;
				onbyte = 0;
				if (mw->wavefile.bps == 1) { // 8 bit
					if (WaveFormat.nChannels == 1) { // mono
						while (y > onbyte) {
							// swap buffer data
							r = databuffer[y];
							databuffer[y] = databuffer[onbyte];
							databuffer[onbyte] = r;
							onbyte += 1;
							y -= 1;

						}
					} else {
						while (y > onbyte) {
							//swap data
							r = databuffer[y];
							l = databuffer[y-1];
							databuffer[y] = databuffer[onbyte+1];
							databuffer[y-1] = databuffer[onbyte];
							databuffer[onbyte] = l;
							databuffer[onbyte+1] = r;
							onbyte += 2;
							y -= 2;
						}
					}
				} else if (mw->wavefile.bps == 2) { // 16 bit 
					if (WaveFormat.nChannels == 1) { // mono
						while (y > onbyte) {
							// swap data
							l = databuffer[y];
							l2 = databuffer[y-1];
							databuffer[y] = databuffer[onbyte+1];
							databuffer[y-1] = databuffer[onbyte];
							databuffer[onbyte] = l2;
							databuffer[onbyte+1] = l;
							onbyte += 2;
							y -= 2;
						}
					} else {
						while (y > onbyte) {
							//swap data
							l = databuffer[y];
							l2 = databuffer[y-1];
							r = databuffer[y-2];
							r2 = databuffer[y-3];
							databuffer[y] = databuffer[onbyte+3];
							databuffer[y-1] = databuffer[onbyte+2];
							databuffer[y-2] = databuffer[onbyte+1];
							databuffer[y-3] = databuffer[onbyte];
							databuffer[onbyte+3] = l;
							databuffer[onbyte+2] = l2;
							databuffer[onbyte+1] = r;
							databuffer[onbyte] = r2;
							onbyte += 4;
							y -= 4;
						}
					}
				}
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				bytestoread -= buffersize;
				inf.seekg((mw->FileSelStart + bytestoread), ios::beg);
				inf.read((char *) databuffer, buffersize);
				y = buffersize - 1;
				onbyte = 0;
				if (mw->wavefile.bps == 1) { // 8 bit
					if (WaveFormat.nChannels == 1) { // mono
						while (y > onbyte) {
							// swap buffer data
							r = databuffer[y];
							databuffer[y] = databuffer[onbyte];
							databuffer[onbyte] = r;
							onbyte += 1;
							y -= 1;

						}
					} else {
						while (y > onbyte) {
							//swap data
							r = databuffer[y];
							l = databuffer[y-1];
							databuffer[y] = databuffer[onbyte+1];
							databuffer[y-1] = databuffer[onbyte];
							databuffer[onbyte] = l;
							databuffer[onbyte+1] = r;
							onbyte += 2;
							y -= 2;
						}
					}
				} else if (mw->wavefile.bps == 2) { // 16 bit 
					if (WaveFormat.nChannels == 1) { // mono
						while (y > onbyte) {
							// swap data
							l = databuffer[y];
							l2 = databuffer[y-1];
							databuffer[y] = databuffer[onbyte+1];
							databuffer[y-1] = databuffer[onbyte];
							databuffer[onbyte] = l2;
							databuffer[onbyte+1] = l;
							onbyte += 2;
							y -= 2;
						}
					} else {
						while (y > onbyte) {
							//swap data
							l = databuffer[y];
							l2 = databuffer[y-1];
							r = databuffer[y-2];
							r2 = databuffer[y-3];
							databuffer[y] = databuffer[onbyte+3];
							databuffer[y-1] = databuffer[onbyte+2];
							databuffer[y-2] = databuffer[onbyte+1];
							databuffer[y-3] = databuffer[onbyte];
							databuffer[onbyte+3] = l;
							databuffer[onbyte+2] = l2;
							databuffer[onbyte+1] = r;
							databuffer[onbyte] = r2;
							onbyte += 4;
							y -= 4;
						}
					}
				}
				outf.write((char *) databuffer, buffersize);
			} 
		}
	}

	inf.seekg(mw->FileSelEnd, ios::beg);
	bytestoread = (mw->wavefile.datasize + mw->FileDataStartPos) - mw->FileSelEnd;
	if (bytestoread > 0) {
		while (!inf.eof()) {
			if (bytestoread < buffersize) {
				inf.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				outf.write((char *) databuffer, buffersize);
			} 
			bytestoread -= buffersize;
		}
	}
	inf.close();
	outf.close();
	
	free(databuffer);

	DeleteFile(mw->wavefile.modfile);
	CopyFile(mw->wavefile.tmpfile, mw->wavefile.modfile, false);
	DeleteFile(mw->wavefile.tmpfile);
	mw->statusstate = "";
	return;
}



/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function swaps the channels of the selected
/*		portion of the file.  
/*
/*************************************************************/
/*************************************************************/
void SwapChannels() {
	if (!mw->fileopen) return;
	mw->statusstate = "Swapping Channels...";
	fstream inf, outf;
	DeleteFile(mw->wavefile.tmpfile);
	inf.open(mw->wavefile.modfile, ios::in | ios::binary);
	outf.open(mw->wavefile.tmpfile, ios::out | ios::binary);
	
	unsigned int buffersize = 10000;
	char *databuffer = (char *)malloc(buffersize);

	DWORD i = 0;
	DWORD bytesread = 0;
	DWORD bytestoread = mw->FileSelStart;

	if (bytestoread > 0) {
		while (!inf.eof()) {
			if (bytestoread < buffersize) {	
				inf.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				outf.write((char *) databuffer, buffersize);
			}		
			bytestoread -= buffersize;
		}
	}

	BYTE r, l, l2;
	r = l = 0;
	bytestoread = mw->FileSelEnd - mw->FileSelStart;

	DWORD y;

	inf.seekg(mw->FileSelStart, ios::beg);

	if (bytestoread > 0) {
		while (!inf.eof()) {
			if (bytestoread < buffersize) {
				
				inf.read((char *) databuffer, bytestoread);
				y = 0;
				if (mw->wavefile.bps == 1) { // 8 bit
					if (WaveFormat.nChannels == 2) { // stereo
						while (y < bytestoread) {
							//swap channels
							l = databuffer[y];
							databuffer[y] = databuffer[y+1];
							databuffer[y+1] = l;
							y += 2;
						}
					}
				} else if (mw->wavefile.bps == 2) { // 16 bit 
					if (WaveFormat.nChannels == 2) { // stereo
				
						while (y < (buffersize - 4)) {
							//swap data
							l = databuffer[y]; //low byte
							l2 = databuffer[y+1]; //high byte
							databuffer[y] = databuffer[y+2]; // low byte
							databuffer[y+1] = databuffer[y+3]; // high byte
							databuffer[y+2] = l;	// low byte
							databuffer[y+3] = l2;  // high byte
							y +=4;
						}
					}
				}
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				y = 0;
				if (mw->wavefile.bps == 1) { // 8 bit
					if (WaveFormat.nChannels == 2) { // stereo
						while (y < (buffersize - 2)) {
							//swap channels
							l = databuffer[y];
							databuffer[y] = databuffer[y+1];
							databuffer[y+1] = l;
							y += 2;

						}
					}
				} else if (mw->wavefile.bps == 2) { // 16 bit 
					if (WaveFormat.nChannels == 2) { // stereo
						while (y < (buffersize - 4)) {
							//swap data
							l = databuffer[y]; //low byte
							l2 = databuffer[y+1]; //high byte
							databuffer[y] = databuffer[y+2]; // low byte
							databuffer[y+1] = databuffer[y+3]; // high byte
							databuffer[y+2] = l;	// low byte
							databuffer[y+3] = l2;  // high byte
							y += 4;
						}
					}
				}
				outf.write((char *) databuffer, buffersize);
				bytestoread -= buffersize;
			}
		}
	}

	inf.seekg(mw->FileSelEnd, ios::beg);
	bytestoread = (mw->wavefile.datasize + mw->FileDataStartPos) - mw->FileSelEnd;
	if (bytestoread > 0) {
		while (!inf.eof()) {
			if (bytestoread < buffersize) {
				inf.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				outf.write((char *) databuffer, buffersize);
			} 
			bytestoread -= buffersize;
		}
	}
	inf.close();
	outf.close();

	free(databuffer);

	DeleteFile(mw->wavefile.modfile);
	CopyFile(mw->wavefile.tmpfile, mw->wavefile.modfile, false);
	DeleteFile(mw->wavefile.tmpfile);
	mw->statusstate = "";
	return;
}



/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description: 
/*		This function fades in the selected portion of the file
/*		from silence to full volume.
/*
/*************************************************************/
/*************************************************************/
void FadeIn() {
	if (!mw->fileopen) return;
	mw->statusstate = "Fading In...";
	fstream inf, outf;
	DeleteFile(mw->wavefile.tmpfile);
	inf.open(mw->wavefile.modfile, ios::in | ios::binary);
	outf.open(mw->wavefile.tmpfile, ios::out | ios::binary);

	unsigned int buffersize = 10000;
	char *databuffer = (char *)malloc(buffersize);

	DWORD i = 0;
	DWORD bytesread = 0;
	DWORD bytestoread = mw->FileSelStart;

	if (bytestoread > 0) {
		while (!inf.eof()) {
			if (bytestoread < buffersize) {	
				inf.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				outf.write((char *) databuffer, buffersize);
			}		
			bytestoread -= buffersize;
		}
	}

	BYTE r, l;
	r = l = 0;
	bytestoread = mw->FileSelEnd - mw->FileSelStart;

	float dec = 0;
	float mul = 0;
	dec = (float)((float) 1 / (float)bytestoread);

	DWORD y;
	inf.seekg(mw->FileSelStart, ios::beg);
	if (bytestoread > 0) {
		while (!inf.eof()) {
			if (bytestoread < buffersize) {
				inf.read((char *) databuffer, bytestoread);
				y = 0;
				if (mw->wavefile.bps == 1) { // 8 bit
					while (y < bytestoread) {
						
						databuffer[y] = (BYTE)((((BYTE)databuffer[y] - (BYTE)127) * (float)mul) + 127);
						y++;
						mul += dec;
					}
				} else if (mw->wavefile.bps == 2) { // 16 bit 
					//not supported
				}
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				y = 0;
				if (mw->wavefile.bps == 1) { // 8 bit
					while (y < buffersize) {
						databuffer[y] = (BYTE)((((BYTE)databuffer[y] - (BYTE)127) * (float)mul) + 127);
						mul += dec;
						y++;
					}
				} else if (mw->wavefile.bps == 2) { // 16 bit 
					//not supported
				}
				outf.write((char *) databuffer, buffersize);
				bytestoread -= buffersize;
			}
		}
	}
	inf.seekg(mw->FileSelEnd, ios::beg);
	bytestoread = (mw->wavefile.datasize + mw->FileDataStartPos) - mw->FileSelEnd;
	if (bytestoread > 0) {
		while (!inf.eof()) {
			if (bytestoread < buffersize) {
				inf.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				outf.write((char *) databuffer, buffersize);
			} 
			bytestoread -= buffersize;
		}
	}
	inf.close();
	outf.close();
	
	free(databuffer);

	DeleteFile(mw->wavefile.modfile);
	CopyFile(mw->wavefile.tmpfile, mw->wavefile.modfile, false);
	DeleteFile(mw->wavefile.tmpfile);
	mw->statusstate = "";
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function fades out the selected portion of the 
/*		file from full volume to silence.
/*
/*************************************************************/
/*************************************************************/
void FadeOut() {
	if (!mw->fileopen) return;
	mw->statusstate = "Fading Out...";
	fstream inf, outf;
	DeleteFile(mw->wavefile.tmpfile);
	inf.open(mw->wavefile.modfile, ios::in | ios::binary);
	outf.open(mw->wavefile.tmpfile, ios::out | ios::binary);
	
	unsigned int buffersize = 10000;
	char *databuffer = (char *)malloc(buffersize);

	DWORD i = 0;
	DWORD bytesread = 0;
	DWORD bytestoread = mw->FileSelStart;

	if (bytestoread > 0) {
		while (!inf.eof()) {
			if (bytestoread < buffersize) {	
				inf.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				outf.write((char *) databuffer, buffersize);
			}		
			bytestoread -= buffersize;
		}
	}

	BYTE r, l;
	r = l = 0;
	bytestoread = mw->FileSelEnd - mw->FileSelStart;

	float dec = 0;
	float mul = 1;
	dec = (float)((float) 1 / (float)(bytestoread));

	DWORD y;
	inf.seekg(mw->FileSelStart, ios::beg);
	int x = 0;
	if (bytestoread > 0) {
		while (!inf.eof()) {
			if (bytestoread < buffersize) {
				inf.read((char *) databuffer, bytestoread);
				y = 0;
				if (mw->wavefile.bps == 1) { // 8 bit
					while (y < bytestoread) {
						databuffer[y] = (BYTE)((((BYTE)databuffer[y] - (BYTE)127) * (float)mul) + 127);
						y++;
						mul -= dec;
					}
				} else if (mw->wavefile.bps == 2) { // 16 bit 
					//not supported
				}
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				y = 0;
				if (mw->wavefile.bps == 1) { // 8 bit
					while (y < buffersize) {
						databuffer[y] = (BYTE)((((BYTE)databuffer[y] - (BYTE)127) * (float)mul) + 127);
						mul -= dec;
						y++;

					}
				} else if (mw->wavefile.bps == 2) { // 16 bit 
					//not supported
				}
				outf.write((char *) databuffer, buffersize);
				bytestoread -= buffersize;
			}
		}
	}
	inf.seekg(mw->FileSelEnd, ios::beg);
	bytestoread = (mw->wavefile.datasize + mw->FileDataStartPos) - mw->FileSelEnd;
	if (bytestoread > 0) {
		while (!inf.eof()) {
			if (bytestoread < buffersize) {
				inf.read((char *) databuffer, bytestoread);
				outf.write((char *) databuffer, bytestoread);
				break;
			} else {
				inf.read((char *) databuffer, buffersize);
				outf.write((char *) databuffer, buffersize);
			} 
			bytestoread -= buffersize;
		}
	}
	inf.close();
	outf.close();
	
	free(databuffer);

	DeleteFile(mw->wavefile.modfile);
	CopyFile(mw->wavefile.tmpfile, mw->wavefile.modfile, false);
	DeleteFile(mw->wavefile.tmpfile);
	mw->statusstate = "";
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function generates a pure wave of the specified
/*		length, amplitude, and frequency.
/*
/*************************************************************/
/*************************************************************/
void GenerateSound() {
	fstream outf;
	mw->pw.samplerate = 44100;
	int counter = 0;
	CString fnn = "";
	CString filename = "";
	CString filepath = "";
	CString fns = "";

	mw->pw.count = 1;

	DWORD dwSizeOfWaveformChunk = (DWORD) (36 + mw->pw.length * mw->pw.samplerate);
	DWORD dwSizeOfFormatChunk = (DWORD)16;
	WORD wFormatTag = (WORD)1;
	WORD wChannels = (WORD)1;
	DWORD dwSamplesPerSecond = (DWORD) mw->pw.samplerate;
	DWORD dwAvgBytesPerSecond = (DWORD) mw->pw.samplerate;
	WORD wBlockAlign = (WORD)1;
	WORD wBitsPerSample = (WORD)8;
	DWORD dwSizeOfWaveFormData = (DWORD) mw->pw.samplerate * mw->pw.length;

	char *data = (char *)malloc(mw->pw.samplerate * mw->pw.length);
	double fangle = 0;

	filepath = "c:\\";
	char t[20];
	itoa(mw->pw.freq, t, 10);
	filename = t;
	filename += "hz.wav";
	fns = filename;
	filename = filepath + filename;
	DeleteFile(filename);
	outf.open(filename, ios::out | ios::binary);

	/////////////////////////////////////////////////////
	//////////// start waveform header //////////////////
	/////////////////////////////////////////////////////
	outf.write("RIFF", 4);
	outf.write((char *) &dwSizeOfWaveformChunk, 4); 
	outf.write("WAVE", 4);
	outf.write("fmt ", 4);
	outf.write((char *) &dwSizeOfFormatChunk, 4);
	outf.write((char *) &wFormatTag, 2);
	outf.write((char *) &wChannels, 2);
	outf.write((char *) &dwSamplesPerSecond, 4);
	outf.write((char *) &dwAvgBytesPerSecond, 4);
	outf.write((char *) &wBlockAlign, 2);
	outf.write((char *) &wBitsPerSample, 2);
	outf.write("data", 4);
	outf.write((char *) &dwSizeOfWaveFormData, 4);
	/////////////////////////////////////////////////////
	////////////// end waveform header //////////////////
	/////////////////////////////////////////////////////

	for (int p = 0; p < mw->pw.length; p++) {
		for (DWORD i = 0; i < (mw->pw.samplerate); i++) {
			data[i] = (BYTE)(127 + (mw->pw.amp * sin(fangle)));
			fangle += 2*PI * mw->pw.freq / 44100;
			if (fangle > 2*PI) {
				fangle -= 2*PI;
			}
		}
		outf.write((char *) data, mw->pw.samplerate);
	}
	outf.close();

	free(data);

	mw->OnCloseFile();
	mw->wavefile.soundfilepath = filename;
	mw->wavefile.soundfilename = fns;
	mw->FindFolders();
	CopyFile(filename, mw->wavefile.modfile, false);
	mw->OpenFile();

	///////enable and disable buttons///////
	mw->ToolBar.EnableButton(IDC_PLAY,TRUE);
	mw->ToolBar.EnableButton(IDC_REWIND,TRUE);
	mw->ToolBar.EnableButton(IDC_FASTFWD,TRUE);
	mw->ToolBar.EnableButton(IDC_STOP,FALSE);
	mw->ToolBar.EnableButton(IDC_GOTOEND,TRUE);
	mw->ToolBar.EnableButton(IDC_GOTOSTART,TRUE);
	mw->ToolBar.EnableButton(IDC_PAUSE,FALSE);
	////////////////////////////////////////
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function overlays a new frequency over the current
/*		file.
/*
/*************************************************************/
/*************************************************************/
void OverlaySound() {
	if (!mw->fileopen) return;
	mw->pw.count++;
	mw->statusstate = "Overlaying Sound...";
	fstream inf, outf;
	
	DeleteFile(mw->wavefile.tmpfile);
	inf.open(mw->wavefile.modfile, ios::in | ios::binary);
	outf.open(mw->wavefile.tmpfile, ios::out | ios::binary);
	
	unsigned int buffersize = 10000;
	char *databuffer = (char *)malloc(buffersize);

	DWORD i = 0;
	DWORD bytestoread = mw->wavefile.datasize;

	inf.read((char *) databuffer, 44);
	outf.write((char *) databuffer, 44); 
	double fangle = 0;


	while (bytestoread > 0) {
		if (bytestoread < buffersize) {
			inf.read((char *) databuffer, bytestoread);
			i = 0;
			while (i < bytestoread) {
				databuffer[i] = (BYTE)((((int)(mw->pw.amp * sin(fangle)) + (int)(((int)(BYTE)databuffer[i] - 127)) * mw->pw.count) / (mw->pw.count+1)) + 127);
				fangle += 2*PI * mw->pw.freq / 44100;
				if (fangle > 2*PI) {
					fangle -= 2*PI;
				}
				i++;
			}
			outf.write((char *) databuffer, bytestoread);
			break;
		} else {
			inf.read((char *) databuffer, buffersize);
			i = 0;
			while (i < buffersize) {
				databuffer[i] = (BYTE)((((int)(mw->pw.amp * sin(fangle)) + (int)(((int)(BYTE)databuffer[i] - 127)) * mw->pw.count) / (mw->pw.count+1)) + 127);
				fangle += 2*PI * mw->pw.freq / 44100;
				if (fangle > 2*PI) {
					fangle -= 2*PI;
				}
				i++;
			}
			outf.write((char *) databuffer, buffersize);
		}
		bytestoread -= buffersize;
	}

	outf.close();

	free(databuffer);

	DeleteFile(mw->wavefile.modfile);
	CopyFile(mw->wavefile.tmpfile, mw->wavefile.modfile, false);
	DeleteFile(mw->wavefile.tmpfile);
	mw->statusstate = "";
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: always returns 0
/*
/*  Description:
/*		This function is the start of the thread that
/*		copies the data to the clipboard.
/*
/*************************************************************/
/*************************************************************/
UINT WINAPI CopyThreadFcn() {
	if (mw->working) return 0;
	mw->working = true;
	if (mw->startselection != mw->endselection) {
		SaveData();
	} else {
		mw->statusstate = "Nothing to Copy";
	}

	mw->working = false;
	return 0;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: always returns 0
/*
/*  Description:
/*		This function is the start of the thread that
/*		cuts the data from the file.
/*
/*************************************************************/
/*************************************************************/
UINT WINAPI CutThreadFcn() {
	if (mw->working) return 0;
	mw->working = true;
	if (mw->startselection != mw->endselection) {
		SaveUndoData();
		SaveData();
		Cut();
		mw->endselection = mw->startselection = -1;
		mw->DeletePosLineList();
		mw->posline = NULL;
		mw->linepos = -1;
		mw->changedsincesaved = true;
		ReloadFile();
	} else {
		mw->statusstate = "Nothing to Cut";
	}
	mw->working = false;
	return 0;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: always returns 0
/*
/*  Description:
/*		This function is the start of the thread that
/*		pastes the data from the clipboard to the file.
/*
/*************************************************************/
/*************************************************************/
UINT WINAPI PasteThreadFcn() {
	if (mw->working) return 0;
	mw->working = true;
	if (mw->SizeOfClipboardData > 0) {
		SaveUndoData();
		Paste();
		mw->endselection = mw->startselection = -1;
		mw->DeletePosLineList();
		mw->posline = NULL;
		mw->linepos = -1;
		mw->changedsincesaved = true;
		ReloadFile();
	} else {
		mw->statusstate = "Nothing to Paste";
	}
	mw->working = false;
	return 0;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: always returns 0
/*
/*  Description:
/*		This function is the start of the thread that
/*		clears the selected portion of the file.
/*
/*************************************************************/
/*************************************************************/
UINT WINAPI ClearThreadFcn() {
	if (mw->working) return 0;
	mw->working = true;
	if (mw->startselection != mw->endselection) {
		SaveUndoData();
		Clear();
		mw->DeletePosLineList();
		mw->posline = NULL;
		mw->linepos = -1;
		mw->changedsincesaved = true;
		ReloadFile();
	} else {
		mw->statusstate = "Nothing to Clear";
	}
	mw->working = false;
	return 0;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: always returns 0
/*
/*  Description:
/*		This function is the start of the thread that
/*		crops the selected portion of the file.
/*
/*************************************************************/
/*************************************************************/
UINT WINAPI CropThreadFcn() {
	if (mw->working) return 0;
	mw->working = true;
	if (mw->startselection != mw->endselection) {
		SaveUndoData();
		Crop();
		mw->endselection = mw->startselection = -1;
		mw->DeletePosLineList();
		mw->posline = NULL;
		mw->linepos = -1;
		mw->changedsincesaved = true;
		ReloadFile();
	} else {
		mw->statusstate = "Nothing to Crop";
	}
	mw->working = false;
	return 0;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: always returns 0
/*
/*  Description:
/*		This function is the start of the thread that
/*		inserts silence into the file.
/*
/*************************************************************/
/*************************************************************/
UINT WINAPI SilenceThreadFcn() {
	if (mw->working) return 0;
	mw->working = true;
	SaveUndoData();
	InsertSilence();
	mw->endselection = mw->startselection = -1;
	mw->DeletePosLineList();
	mw->posline = NULL;
	mw->linepos = -1;
	mw->changedsincesaved = true;
	ReloadFile();
	mw->working = false;
	return 0;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: always returns 0
/*
/*  Description:
/*		This function is the start of the thread that
/*		copies the selected portion of the file to a new file.
/*
/*************************************************************/
/*************************************************************/
UINT WINAPI CopyToNewThreadFcn() {
	if (mw->working) return 0;
	mw->working = true;
	if (mw->startselection != mw->endselection) {
		CopyToNew();
	} else {
		mw->statusstate = "Nothing to Copy";
	}
	mw->working = false;
	return 0;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: always returns 0
/*
/*  Description:
/*		This function is the start of the thread that
/*		pastes the copied data to a new file.
/*
/*************************************************************/
/*************************************************************/
UINT WINAPI PasteToNewThreadFcn() {
	if (mw->working) return 0;
	mw->working = true;
	if (mw->SizeOfClipboardData > 0) {
		PasteToNew();
	} else {
		mw->statusstate = "Nothing to Paste";
	}
	mw->working = false;
	return 0;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: always returns 0
/*
/*  Description:
/*		This function is the start of the thread that
/*		reverses the selected portion of the file.
/*
/*************************************************************/
/*************************************************************/
UINT WINAPI ReverseThreadFcn() {
	if (mw->working) return 0;
	mw->working = true;
	if (mw->startselection != mw->endselection) {
		SaveUndoData();
		Reverse();
//		mw->endselection = mw->startselection = -1;
		mw->DeletePosLineList();
		mw->posline = NULL;
		mw->linepos = -1;
		mw->changedsincesaved = true;
		ReloadFile();
	} else {
		mw->statusstate = "Nothing to Reverse";
	}
	mw->working = false;
	return 0;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: always returns 0
/*
/*  Description:
/*		This function is the start of the thread that
/*		swaps the channels of the selected portion of the file.
/*
/*************************************************************/
/*************************************************************/
UINT WINAPI SwapChannelsThreadFcn() {
	if (mw->working) return 0;
	mw->working = true;
	if (mw->startselection != mw->endselection) {
		SaveUndoData();
		SwapChannels();
		mw->DeletePosLineList();
		mw->posline = NULL;
		mw->linepos = -1;
		mw->changedsincesaved = true;
		ReloadFile();
	} else {
		mw->statusstate = "Nothing to Swap";
	}
	mw->working = false;
	return 0;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: always returns 0
/*
/*  Description:
/*		This function is the start of the thread that
/*		fades in the selected portion of the file.
/*
/*************************************************************/
/*************************************************************/
UINT WINAPI FadeInThreadFcn() {
	if (mw->working) return 0;
	mw->working = true;
	if (mw->startselection != mw->endselection) {
		SaveUndoData();
		FadeIn();
//		mw->endselection = mw->startselection = -1;
		mw->DeletePosLineList();
		mw->posline = NULL;
		mw->linepos = -1;
		mw->changedsincesaved = true;
		ReloadFile();
	} else {
		mw->statusstate = "Nothing to Fade In";
	}
	mw->working = false;
	return 0;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: always returns 0
/*
/*  Description:
/*		This function is the start of the thread that
/*		fades out the selected portion of the file.
/*
/*************************************************************/
/*************************************************************/
UINT WINAPI FadeOutThreadFcn() {
	if (mw->working) return 0;
	mw->working = true;
	if (mw->startselection != mw->endselection) {
		SaveUndoData();
		FadeOut();
		mw->DeletePosLineList();
		mw->posline = NULL;
		mw->linepos = -1;
		mw->changedsincesaved = true;
		ReloadFile();
	} else {
		mw->statusstate = "Nothing to Fade Out";
	}
	mw->working = false;
	return 0;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: always returns 0
/*
/*  Description:
/*		This function is the start of the thread that
/*		reads the header of the wave file.
/*
/*************************************************************/
/*************************************************************/
UINT WINAPI GetWaveInfoThreadFcn() {
	if (mw->working) return 0;
	mw->working = true;
	mw->GetWaveInfo();
	mw->working = false;
	return 0;
}



/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: always returns 0
/*
/*  Description:
/*		This function is the start of the thread that
/*		generates the pure wave sound file.
/*		
/*
/*************************************************************/
/*************************************************************/
UINT WINAPI GenerateThreadFcn() {
	if (mw->working) return 0;
	mw->working = true;
	GenerateSound();
	mw->working = false;
	return 0;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: always returns 0
/*
/*  Description:
/*		This function is the start of the thread that
/*		overlays a new frequency over the open file.
/*
/*************************************************************/
/*************************************************************/
UINT WINAPI OverlayThreadFcn() {
	if (mw->working) return 0;
	mw->working = true;
	SaveUndoData();
	mw->changedsincesaved = true;
	OverlaySound();
	ReloadFile();
	mw->working = false;

	return 0;
}




/*************************************************************/
/*************************************************************/
/*
/*  Parameters: x - which thread to create
/*
/*  Returns: Handle to the thread created
/*
/*  Description:
/*		This function creates the thread specified by x to do
/*		its task.  Threads are used so that the work of the 
/*		program does not freeze the window.
/*
/*************************************************************/
/*************************************************************/
HANDLE CreateMyThread(CString x) {
	if (x == "getwaveinfo") {
		return CreateThread(NULL, 0, (unsigned long (__stdcall *)(void *)) &GetWaveInfoThreadFcn, NULL, 0, &mw->dwThreadID);
	} else if (x == "copy") {
		return CreateThread(NULL, 0, (unsigned long (__stdcall *)(void *)) &CopyThreadFcn, NULL, 0, &mw->dwThreadID);
	} else if (x == "cut") {
		return CreateThread(NULL, 0, (unsigned long (__stdcall *)(void *)) &CutThreadFcn, NULL, 0, &mw->dwThreadID);
	} else if (x == "paste") {
		return CreateThread(NULL, 0, (unsigned long (__stdcall *)(void *)) &PasteThreadFcn, NULL, 0, &mw->dwThreadID);
	} else if (x == "clear") {
		return CreateThread(NULL, 0, (unsigned long (__stdcall *)(void *)) &ClearThreadFcn, NULL, 0, &mw->dwThreadID);
	} else if (x == "crop") {
		return CreateThread(NULL, 0, (unsigned long (__stdcall *)(void *)) &CropThreadFcn, NULL, 0, &mw->dwThreadID);
	} else if (x == "silence") {
		return CreateThread(NULL, 0, (unsigned long (__stdcall *)(void *)) &SilenceThreadFcn, NULL, 0, &mw->dwThreadID);
	} else if (x == "copytonew") {
		return CreateThread(NULL, 0, (unsigned long (__stdcall *)(void *)) &CopyToNewThreadFcn, NULL, 0, &mw->dwThreadID);
	} else if (x == "pastetonew") {
		return CreateThread(NULL, 0, (unsigned long (__stdcall *)(void *)) &PasteToNewThreadFcn, NULL, 0, &mw->dwThreadID);
	} else if (x == "reverse") {
		return CreateThread(NULL, 0, (unsigned long (__stdcall *)(void *)) &ReverseThreadFcn, NULL, 0, &mw->dwThreadID);
	} else if (x == "swapchannels") {
		return CreateThread(NULL, 0, (unsigned long (__stdcall *)(void *)) &SwapChannelsThreadFcn, NULL, 0, &mw->dwThreadID);
	} else if (x == "fadein") {
		return CreateThread(NULL, 0, (unsigned long (__stdcall *)(void *)) &FadeInThreadFcn, NULL, 0, &mw->dwThreadID);
	} else if (x == "fadeout") {
		return CreateThread(NULL, 0, (unsigned long (__stdcall *)(void *)) &FadeOutThreadFcn, NULL, 0, &mw->dwThreadID);
	} else if (x == "generate") {
		return CreateThread(NULL, 0, (unsigned long (__stdcall *)(void *)) &GenerateThreadFcn, NULL, 0, &mw->dwThreadID);
	} else if (x == "overlay") {
		return CreateThread(NULL, 0, (unsigned long (__stdcall *)(void *)) &OverlayThreadFcn, NULL, 0, &mw->dwThreadID);
	}
	return NULL;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function starts the creation of the thread that
/*		cuts the data from the file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnCut(){
	if ((working) || (playing) || (paused) || (!fileopen)) return;
	CreateMyThread("cut");
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function starts the creation of the thread that
/*		copies the data from the file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnCopy(){
	if ((working) || (playing) || (paused) || (!fileopen)) return;
	CreateMyThread("copy");
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function starts the creation of the thread that
/*		pastes the data to the file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnPaste(){
	if ((working) || (playing) || (paused) || (!fileopen)) return;
	CreateMyThread("paste");
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function starts the creation of the thread that
/*		clears the data on the file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnClear(){
	if ((working) || (playing) || (paused) || (!fileopen)) return;
	CreateMyThread("clear");
	return;
}



/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function starts the creation of the thread that
/*		crops the data on the file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnCrop() {
	if ((working) || (playing) || (paused) || (!fileopen)) return;
	CreateMyThread("crop");
	return;
}



/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function starts the creation of the thread that
/*		inserts silence into the file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnInsertSilence(){
	if ((working) || (playing) || (paused) || (!fileopen)) return;
	SilenceDialog dg("SilenceDialog");
	dg.DoModal();
	CreateMyThread("silence");
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function starts the creation of the thread that
/*		copies the data from the file to a new file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnCopyToNew() {
	if ((working) || (playing) || (paused) || (!fileopen)) return;
	CreateMyThread("copytonew");
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function starts the creation of the thread that
/*		pastes the copied data to a new file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnPasteToNew() {
	if ((working) || (playing) || (paused) || (!fileopen)) return;
	CreateMyThread("pastetonew");
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function starts the creation of the thread that
/*		reverses the selected portion of the file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnReverse() {
	if ((working) || (playing) || (paused) || (!fileopen)) return;
	CreateMyThread("reverse");
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function starts the creation of the thread that
/*		swaps the channels of the selected portion of the file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnSwapChannels() {
	if ((working) || (playing) || (paused) || (!fileopen)) return;
	CreateMyThread("swapchannels");
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function starts the creation of the thread that
/*		fades in the selected portion of the file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnFadeIn() {
	if ((working) || (playing) || (paused) || (!fileopen)) return;
	CreateMyThread("fadein");
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function starts the creation of the thread that
/*		fades out the selected portion of the file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnFadeOut() {
	if ((working) || (playing) || (paused) || (!fileopen)) return;
	CreateMyThread("fadeout");
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: None
/*
/*  Returns: None
/*
/*  Description:
/*		This function pops up the info dialog box to display
/*		information about the file.
/*		
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnInfo(){
	if (fileopen) {
		InfoDialog dg("InfoDialog");
		dg.DoModal();
	} else {
		MessageBox("No File Information Available!", "No File Info");
	}
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function undoes the last changes made to the file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnUndo() {
	if ((working) || (playing) || (paused) || (!fileopen)) return;

	mw->statusstate = "Undo...";
	working = true;
	CopyFile(wavefile.modfile, wavefile.tmpfile, false);
	DeleteFile(wavefile.modfile);
	CopyFile(wavefile.undofile, wavefile.modfile, false);
	DeleteFile(wavefile.undofile);
	CopyFile(wavefile.tmpfile, wavefile.undofile, false);
	DeleteFile(wavefile.tmpfile);
	ReloadFile();
	working = false;
	mw->statusstate = "";
 	return;
}



/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function pops up the generate dialog so the user
/*		can enter the information of the sound he/she wants
/*		the program to generate
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnGenerate() {
	GenerateDialog fd("GenerateDialog");
	if (fd.DoModal() == IDCANCEL) return;
	
	pw.length = fd.GetLength();
	pw.freq = fd.GetFrequency();
	pw.amp = fd.GetAmplitude();

	CreateMyThread("generate");
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function pops up the overlay dialog so the user 
/*		can enter the information of the sound he/she wants
/*		the program to overlay onto the current sound.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnOverlay() {
	if ((working) || (playing) || (paused) || (!fileopen)) return;
	OverlayDialog fd("OverlayDialog");
	if (fd.DoModal() == IDCANCEL) return;
	
	pw.freq = fd.GetFrequency();
	pw.amp = fd.GetAmplitude();
	CreateMyThread("overlay");
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function starts playback of the current file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnPlay() {
    MCI_OPEN_PARMS OpenParms;
    MCI_PLAY_PARMS PlayParms;
	MCI_STATUS_PARMS StatusParms;
	MCI_SEEK_PARMS SeekParms;
	MCI_SET_PARMS SetParms;
	if (working) {
		return;
	}
	if (fileopen) {
		if (!playing) {
			OnStop();
			DrawVerticalLine(lastmark);

			///////enable and disable buttons///////
			ToolBar.EnableButton(IDC_RECORD, FALSE);
			ToolBar.EnableButton(IDC_PAUSE,TRUE);
			ToolBar.EnableButton(IDC_STOP, TRUE);
			////////////////////////////////////////

			lastmark = nextmark = -1;
			playing = true;
			stopped = false;
			paused = false;


			OpenParms.dwCallback = 0;
			OpenParms.wDeviceID = 0;
			OpenParms.lpstrDeviceType = "waveaudio";
			OpenParms.lpstrElementName = wavefile.modfile;
			OpenParms.lpstrAlias = NULL;


			mciSendCommand(0, MCI_OPEN, MCI_OPEN_TYPE | MCI_OPEN_ELEMENT, (DWORD)(LPVOID) &OpenParms);

			wDeviceID = OpenParms.wDeviceID;
			PlayParms.dwCallback = (DWORD) this->GetSafeHwnd();
			
			//////////////////////////////////////////////////////////////////////////////////
			//   needs this to get the line to move on the first hit of play....            //
			mciSendCommand(wDeviceID, MCI_PLAY, MCI_NOTIFY, (DWORD)(LPVOID) &PlayParms);	//
			mciSendCommand(wDeviceID, MCI_STOP, 0, NULL);									//
			//////////////////////////////////////////////////////////////////////////////////

			SetParms.dwTimeFormat = MCI_FORMAT_MILLISECONDS;
			SetParms.dwCallback = 0;
			mciSendCommand(wDeviceID, MCI_SET, MCI_SET_TIME_FORMAT, (DWORD)(LPMCI_SET_PARMS) &SetParms);

			StatusParms.dwItem = MCI_STATUS_LENGTH;
			mciSendCommand(wDeviceID, MCI_STATUS, MCI_STATUS_ITEM | MCI_NOTIFY, (DWORD)(LPMCI_STATUS_PARMS) &StatusParms);
			mci_wavelength = StatusParms.dwReturn;


			if (startselection <= endselection) {
				if (startselection != -1) {
					PlayStart = (DWORD)(mci_wavelength * startselection);
				} else {
					PlayStart = 0;
				}
				if (endselection != -1) {
					PlayEnd = (DWORD)(mci_wavelength * endselection);
				} else {
					PlayEnd = mci_wavelength;
				}
				if (PlayStart == PlayEnd) {
					PlayEnd = mci_wavelength;
				}
			} else {
				if (startselection != -1) {
					PlayEnd = (DWORD)(mci_wavelength * startselection);
				} else {
					PlayEnd = 0;
				}
				if (endselection != -1) {
					PlayStart = (DWORD)(mci_wavelength * endselection);
				} else {
					PlayStart = mci_wavelength;
				}
				if (PlayStart == PlayEnd) {
					PlayEnd = mci_wavelength;
				}
			}

			SeekParms.dwCallback = 0;
			SeekParms.dwTo = PlayStart;
			mciSendCommand(wDeviceID, MCI_SEEK, MCI_NOTIFY | MCI_TO, (DWORD)(LPMCI_SEEK_PARMS) &SeekParms);
			mciSendCommand(wDeviceID, MCI_PLAY, MCI_NOTIFY, (DWORD)(LPVOID) &PlayParms);
			statusstate = "Playing";
			ShowStatus();
		} else if (paused) { 
			OnPause();
		} else {
			OnStop();
			OnPlay();
		}
	}
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function stops playback of the current file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnStop() {
	if (working) {
		return;
	}
	if (fileopen) {
		if (playing){
			if (paused) {
				OnPause();
			}
			///////enable and disable buttons///////
			ToolBar.EnableButton(IDC_RECORD, TRUE);
			ToolBar.EnableButton(IDC_PAUSE,FALSE);
			ToolBar.EnableButton(IDC_STOP, FALSE);
			////////////////////////////////////////
			stopped = true;
			playing = false;
			DrawVerticalLine(lastmark);
			lastmark = nextmark = -1;
			mciSendCommand(wDeviceID, MCI_STOP, 0, NULL);
			mciSendCommand(wDeviceID, MCI_CLOSE, 0, NULL);
			statusstate = "Stopped";
			ShowStatus();

		}
	}
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function pauses the playback of the current file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnPause() {
	if (working) {
		return;
	}
	if (fileopen) {
		if (playing) {
			if (!paused) {
				paused = true;
				mciSendCommand(wDeviceID, MCI_PAUSE, 0, NULL);
				statusstate = "Paused";
				ShowStatus();
			} else {
				paused = false;
				mciSendCommand(wDeviceID, MCI_PLAY, 0, NULL);
				statusstate = "Playing";
				ShowStatus();
			}
		}
	}
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function fast forwards the playback of the current
/*		file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnFastFwd() {
	MCI_SEEK_PARMS seekpar;
	MCI_STATUS_PARMS statuspar;
	if ((working) || (!fileopen)){
		return;
	}
	float t = 0;

	if (playing) {
		statuspar.dwCallback = 0;
		statuspar.dwItem = MCI_STATUS_POSITION;
		seekpar.dwCallback = 0;
		mciSendCommand(wDeviceID, MCI_STATUS, MCI_STATUS_ITEM | MCI_WAIT, (DWORD)(LPMCI_STATUS_PARMS) &statuspar);
		seekpar.dwTo = statuspar.dwReturn + 1000;
		mciSendCommand(wDeviceID, MCI_PAUSE, 0, NULL);
		mciSendCommand(wDeviceID, MCI_SEEK, MCI_NOTIFY | MCI_TO, (DWORD)(LPMCI_SEEK_PARMS) &seekpar);
		mciSendCommand(wDeviceID, MCI_PLAY, 0, NULL);
	} else {
		if (startselection == 1) return;
		RestoreLine();
		if ((shiftkey) && (controlkey)) {
			DrawSelection(endselection, 1);
			endselection = 1;
			linepos = startselection;
			if (startselection == endselection) {
				SaveLine();
				DrawLine();
			}
		} else if (shiftkey) {
			t = (float)((float)((float)(tophalf.right - tophalf.left) * (float)endselection) + 1) / (float)(tophalf.right - tophalf.left);
			DrawSelection(endselection, t);
			endselection = t;
			linepos = startselection;
			if (startselection == endselection) {
				SaveLine();
				DrawLine();
			}
		} else if (controlkey) {
			DrawSelection(startselection, endselection);
			linepos = startselection = endselection = 1;
			SaveLine();
			DrawLine();
		} else {
			DrawSelection(startselection, endselection);
			startselection = endselection = (float)((float)((float)(tophalf.right - tophalf.left) * (float)startselection) + 1) / (float)(tophalf.right - tophalf.left);
			linepos = startselection;
			//DrawSelection(startselection, endselection);
			SaveLine();
			DrawLine();	
		}
		CalculateFilePositions();
	}
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function rewinds playback of the current file.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnRewind() {
	MCI_SEEK_PARMS seekpar;
	MCI_STATUS_PARMS statuspar;
	if ((working) || (!fileopen)){
		return;
	}
	float t = 0;

	if (playing) {
		statuspar.dwCallback = 0;
		statuspar.dwItem = MCI_STATUS_POSITION;
		seekpar.dwCallback = 0;
		mciSendCommand(wDeviceID, MCI_STATUS, MCI_STATUS_ITEM | MCI_WAIT, (DWORD)(LPMCI_STATUS_PARMS) &statuspar);
		seekpar.dwTo = statuspar.dwReturn - 1000;
		mciSendCommand(wDeviceID, MCI_PAUSE, 0, NULL);
		mciSendCommand(wDeviceID, MCI_SEEK, MCI_NOTIFY | MCI_TO, (DWORD)(LPMCI_SEEK_PARMS) &seekpar);
		mciSendCommand(wDeviceID, MCI_PLAY, 0, NULL);
	} else {
		if (startselection == 0) return;
		RestoreLine();
		if ((shiftkey) && (controlkey)) {
			RestoreLine();
			DrawSelection(endselection, 0);
			endselection = 0;
			linepos = startselection;
			if (startselection == endselection) {
				SaveLine();
				DrawLine();
			}
		} else if (shiftkey) {
			t = (float)((float)((float)(tophalf.right - tophalf.left) * (float)endselection) - 1) / (float)(tophalf.right - tophalf.left);
			RestoreLine();
			DrawSelection(endselection, t);
			endselection = t;
			linepos = startselection;
			if (startselection == endselection) {
				SaveLine();
				DrawLine();
			}
		} else if (controlkey) {
			DrawSelection(startselection, endselection);
			linepos = startselection = endselection = 0;
			SaveLine();
			DrawLine();
		} else {
			RestoreLine();
			DrawSelection(startselection, endselection);
			startselection = endselection = (float)((float)(((float)(tophalf.right - tophalf.left)) * (float)startselection) - 1) / (float)(tophalf.right - tophalf.left);
			linepos = startselection;
			SaveLine();
			DrawLine();	
		}
		CalculateFilePositions();
	}
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function starts recording from the default device.
/*
/*************************************************************/
/*************************************************************/
void CMainWindow::RecordStart() {
	MCI_OPEN_PARMS OpenParms;
	MCI_RECORD_PARMS RecordParms;

	DWORD err;
	statusstate = "Opening Device...";
	ShowStatus();
	OpenParms.dwCallback = 0;
	OpenParms.wDeviceID = 0;
	OpenParms.lpstrDeviceType = "waveaudio";
	OpenParms.lpstrElementName = "";
	OpenParms.lpstrAlias = NULL;

	err = mciSendCommand(0, MCI_OPEN, MCI_WAIT | MCI_OPEN_TYPE | MCI_OPEN_ELEMENT, (DWORD)(LPMCI_OPEN_PARMS) &OpenParms);
	
	if (err != 0) {
		MessageBox("error starting recording");
		return;
	}

	///////enable and disable buttons///////
	ToolBar.EnableButton(IDC_PLAY,FALSE);
	ToolBar.EnableButton(IDC_REWIND,FALSE);
	ToolBar.EnableButton(IDC_FASTFWD,FALSE);
	ToolBar.EnableButton(IDC_STOP,FALSE);
	ToolBar.EnableButton(IDC_GOTOEND,FALSE);
	ToolBar.EnableButton(IDC_GOTOSTART,FALSE);
	ToolBar.EnableButton(IDC_PAUSE,FALSE);
	////////////////////////////////////////

	wDeviceID = OpenParms.wDeviceID;

	RecordParms.dwCallback = (DWORD) this->GetSafeHwnd();
	RecordParms.dwFrom = 0;
	RecordParms.dwTo = 0;

	mciSendCommand(wDeviceID, MCI_RECORD, MCI_NOTIFY, (DWORD)(LPMCI_RECORD_PARMS) &RecordParms);


	statusstate = "Recording...";
	ShowStatus();
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function stops recording from the default device.
/*
/*************************************************************/
/*************************************************************/
void CMainWindow::RecordEnd() {
	MCI_GENERIC_PARMS GenericParms;
	MCI_SAVE_PARMS SaveParms;
	DWORD err;
	CString recordfile;

	GenericParms.dwCallback = 0;
	
	err = mciSendCommand(wDeviceID, MCI_STOP, MCI_WAIT, (DWORD)(LPMCI_GENERIC_PARMS) &GenericParms);
	if (err != 0) {
		MessageBox("Error");
		return;
	}
	if (wavefile.SoundFileFolder == "") {
		recordfile = "c:\\rec.wav";
	} else {
		recordfile = wavefile.SoundFileFolder + "rec.wav";
	}
	SaveParms.dwCallback = 0;
	SaveParms.lpfilename = recordfile;

	mciSendCommand(wDeviceID, MCI_SAVE, MCI_WAIT | MCI_SAVE_FILE, (DWORD)(LPMCI_SAVE_PARMS) &SaveParms);
	mciSendCommand(wDeviceID, MCI_CLOSE, MCI_WAIT, (DWORD)(LPMCI_GENERIC_PARMS) &GenericParms);
	
	statusstate = "Saving Recording...";
	ShowStatus();
	CFileDialog  * dlg = new CFileDialog (false, NULL, NULL, OFN_EXPLORER | OFN_NONETWORKBUTTON | OFN_HIDEREADONLY, "WAVE Files (*.wav)|*.wav||", this);

saveit:
	if(dlg->DoModal()== IDOK) {
		statusstate = "Writing File...";
		ShowStatus();
		CString d = dlg->GetPathName();
		if (!CopyFile(recordfile, d, true)) {
			if (MessageBox("File Already Exists!  Do you want to replace it?", "File Exists", MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2 | MB_APPLMODAL) == IDYES) {
				wavefile.soundfilepath = dlg->GetPathName();
				wavefile.soundfilename = dlg->GetFileName();
				FindFolders();
				ext = dlg->GetFileExt();
				DeleteFile(d);
				CopyFile(recordfile, d, false);
				DeleteFile(recordfile);
				working = true;
				OnCloseFile();
				CopyFile(d, wavefile.modfile, false);
				OpenFile();
				working = false;
			} else {
				goto saveit;
			}
		} else {
			wavefile.soundfilepath = dlg->GetPathName();
			wavefile.soundfilename = dlg->GetFileName();
			FindFolders();
			ext = dlg->GetFileExt();
			DeleteFile(recordfile);
			working = true;
			OnCloseFile();
			CopyFile(d, wavefile.modfile, false);
			OpenFile();
			working = false;

		}
	}
	statusstate = "";
	ShowStatus();

	
	///////enable and disable buttons///////
	ToolBar.EnableButton(IDC_PLAY,TRUE);
	ToolBar.EnableButton(IDC_REWIND,TRUE);
	ToolBar.EnableButton(IDC_FASTFWD,TRUE);
	ToolBar.EnableButton(IDC_STOP,FALSE);
	ToolBar.EnableButton(IDC_GOTOEND,TRUE);
	ToolBar.EnableButton(IDC_GOTOSTART,TRUE);
	////////////////////////////////////////

	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function starts or stops recording.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnRecord() {
	if ((working) || (playing) || (paused)) return;

	if (!recording) {
		recording = true;
		RecordStart();
	} else {
		RecordEnd();
		recording = false;
	}
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function increases the playback volume.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnVolumeUp() {
	if (dwVolume < 0xFFFFFFFF) {
		dwVolume += 0x22222222;
		SetVolume();
	}
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function decreases the playback volume
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnVolumeDown(){
	if (dwVolume > 0x00000000) {
		if (dwVolume <= 0x11111111) {
			dwVolume = 0x00000000;
		} else {
			dwVolume -= 0x22222222;
		}
		SetVolume();
	}
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function sets the volume to 0
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnSetVolume0() {
	dwVolume = 0x00000000;
	SetVolume();
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function sets the volume to 1
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnSetVolume1() {
	dwVolume = 0x11111111;
	SetVolume();
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function sets the volume to 2
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnSetVolume2() {
	dwVolume = 0x33333333;
	SetVolume();
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function sets the volume to 3
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnSetVolume3() {
	dwVolume = 0x55555555;
	SetVolume();
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function sets the volume to 4
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnSetVolume4() {
	dwVolume = 0x77777777;
	SetVolume();
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function sets the volume to 5
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnSetVolume5() {
	dwVolume = 0x99999999;
	SetVolume();
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function sets the volume to 6
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnSetVolume6() {
	dwVolume = 0xBBBBBBBB;
	SetVolume();
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function sets the volume to 7
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnSetVolume7() {
	dwVolume = 0xDDDDDDDD;
	SetVolume();
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function sets the volume to 8
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnSetVolume8() {
	dwVolume = 0xFFFFFFFF;
	SetVolume();
	return;
}



/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function sets playback to repeat
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnLoopPlayback() {
	loop = !loop;
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function turns the right channel on/off.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnRightChannel() {
	rightchannel = !rightchannel;
	SetVolume();
	return;
} 


/*************************************************************/
/*************************************************************/
/*
/*  Parameters:  None
/*
/*  Returns: None
/*
/*  Description:
/*		This function turns the left channel on/off.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnLeftChannel() {
	leftchannel = !leftchannel;
	SetVolume();
	return;
}


/*************************************************************/
/*************************************************************/
/*
/*  Parameters: cm - the current menu item
/*
/*  Returns:None
/*
/*  Description:
/*		Sets the menus to enabled/disabled and
/*		checked/unchecked.
/*
/*************************************************************/
/*************************************************************/
afx_msg void CMainWindow::OnSetMenus(CCmdUI *cm){
	if ((cm->m_nID == IDM_OPEN) || (cm->m_nID == IDM_EXIT)) return;
	if ((cm->m_nID >= 2002) && (cm->m_nID <= 2007)) { // file menu
		if ((!fileopen) || (working) || (playing)) {
			cm->Enable(false);
		} else if (cm->m_nID == IDM_SAVE) {
			if (changedsincesaved) {
				cm->Enable(true);
			} else {
				cm->Enable(false);
			}
		} else {
			if ((!changedsincesaved) && (cm->m_nID == IDM_REVERT)) {
				cm->Enable(false);
			} else {
				cm->Enable(true);
			}
		}
	} else if ((cm->m_nID >= IDM_UNDO) && (cm->m_nID <= IDM_SWAPCHANNELS)) {
		if ((working) || (playing) || (!fileopen)){
			cm->Enable(false);
		} else if ((cm->m_nID == IDM_FADEIN) || (cm->m_nID == IDM_FADEOUT)) {
			if (wavefile.bps == 2) {
				cm->Enable(false);
			} else {
				cm->Enable(true);
			}			
		} else if (cm->m_nID == IDM_UNDO) {
			if (changedsincesaved) {
				cm->Enable(true);			
			} else {
				cm->Enable(false);
			}
		} else if ((startselection == endselection)) {
			if (cm->m_nID == IDM_COPYTONEW) {
				cm->Enable(false);
			} else if ((cm->m_nID == IDM_PASTE) || (cm->m_nID == IDM_PASTETONEW) || (cm->m_nID == IDM_INSERTSILENCE)) {
				if (SizeOfClipboardData > 0) {
					cm->Enable(true);
				} else if (cm->m_nID == IDM_INSERTSILENCE) {
					cm->Enable(true);
				} else {
					cm->Enable(false);
				}
			}
		} else if ((startselection != endselection)) {
			if ((cm->m_nID == IDM_PASTE) || (cm->m_nID == IDM_PASTETONEW)) {
				if (SizeOfClipboardData > 0) {
					cm->Enable(true);
				} else {
					cm->Enable(false);
				}
			}
		}
	} else if ((cm->m_nID >= IDM_SELALL) && (cm->m_nID <= IDM_SELTOSTART)) {
		if (!fileopen) {
			cm->Enable(false);
		} 
	} else if ((cm->m_nID >= IDM_VOLUME0) && (cm->m_nID <= IDM_INFO)) {
		if (!fileopen) {
			if ((cm->m_nID == IDM_LOOPPLAYBACK) || (cm->m_nID == IDM_INFO) || (cm->m_nID == IDM_RIGHTCHANNEL) || (cm->m_nID == IDM_LEFTCHANNEL)) {
				if (cm->m_nID == IDM_RIGHTCHANNEL) {
					if (rightchannel) {
						cm->SetCheck();
					} else {
						cm->SetCheck(0);
					}
				} else if (cm->m_nID == IDM_LEFTCHANNEL) {
					if (leftchannel) {
						cm->SetCheck();
					} else {
						cm->SetCheck(0);
					} 
				} else if (cm->m_nID == IDM_LOOPPLAYBACK) {
					if (loop) {
						cm->SetCheck();
					} else {
						cm->SetCheck(0);
					}
				}
				cm->Enable(false);
			} else {
				if (cm->m_nID == (unsigned int)(IDM_VOLUME0 + volumelevel)) {
					cm->SetCheck();
				} else if ((cm->m_nID <= IDM_VOLUME8) && (cm->m_nID >= IDM_VOLUME0)) {
					cm->SetCheck(0);
				}
				cm->Enable(true);
			}
		} else {
			if (cm->m_nID == (unsigned int)(IDM_VOLUME0 + volumelevel)) {
				cm->SetCheck();
			} else if ((cm->m_nID <= IDM_VOLUME8) && (cm->m_nID >= IDM_VOLUME0)) {
				cm->SetCheck(0);
			} else if ((cm->m_nID == IDM_LOOPPLAYBACK)) {
				if (loop) {
					cm->SetCheck();
				} else {
					cm->SetCheck(0);
				}
			} else if (cm->m_nID == IDM_RIGHTCHANNEL) {
				if (rightchannel) {
					cm->SetCheck();
				} else {
					cm->SetCheck(0);
				}
			} else if (cm->m_nID == IDM_LEFTCHANNEL) {
				if (leftchannel) {
					cm->SetCheck();
				} else {
					cm->SetCheck(0);
				}
			}
			cm->Enable(true);
		}
	} else if (cm->m_nID == IDM_OVERLAY) {
		if (fileopen) {
			cm->Enable(true);
		} else {
			cm->Enable(false);
		}
	}
	return;
}


//////////////message map for main window ///////////////////////
BEGIN_MESSAGE_MAP (CMainWindow, CFrameWnd)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_WM_SETFOCUS()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_MOUSEMOVE()
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	ON_WM_TIMER()
	ON_WM_CLOSE()

	//update menus
	ON_UPDATE_COMMAND_UI_RANGE(IDM_OPEN,IDM_EXIT, OnSetMenus)
	ON_UPDATE_COMMAND_UI_RANGE(IDM_UNDO, IDM_SWAPCHANNELS, OnSetMenus)
	ON_UPDATE_COMMAND_UI_RANGE(IDM_SELALL, IDM_SELTOSTART, OnSetMenus)
	ON_UPDATE_COMMAND_UI_RANGE(IDM_VOLUME0, IDM_INFO, OnSetMenus)
	ON_UPDATE_COMMAND_UI_RANGE(IDM_GENERATE, IDM_OVERLAY, OnSetMenus)

	//File Menu Messages
	ON_COMMAND(IDM_SAVE, OnSave)
	ON_COMMAND(IDM_SAVEAS, OnSaveAs)
	ON_COMMAND(IDM_OPEN, OnOpen)
	ON_COMMAND(IDM_CLOSE, OnCloseFile)
	ON_COMMAND(IDM_EXIT, OnClose)
	ON_COMMAND(IDM_REVERT, OnRevert)
	

	//selection menu messages
	ON_COMMAND(IDM_SELTOEND, OnSelToEnd)
	ON_COMMAND(IDM_SELTOSTART, OnSelToStart)
	ON_COMMAND(IDM_SELNONE, OnSelNone)
	

	//Edit Menu Messages
	ON_COMMAND(IDM_UNDO, OnUndo)
	ON_COMMAND(IDM_SELALL, OnSelAll)
	ON_COMMAND(IDM_CROP, OnCrop)
	ON_COMMAND(IDM_CUT, OnCut)
	ON_COMMAND(IDM_COPY, OnCopy)
	ON_COMMAND(IDM_PASTE, OnPaste)
	ON_COMMAND(IDM_CLEAR, OnClear)
	ON_COMMAND(IDM_REVERSE, OnReverse)
	ON_COMMAND(IDM_SWAPCHANNELS, OnSwapChannels)
	ON_COMMAND(IDM_COPYTONEW, OnCopyToNew)
	ON_COMMAND(IDM_PASTETONEW, OnPasteToNew)
	ON_COMMAND(IDM_INSERTSILENCE, OnInsertSilence)
	ON_COMMAND(IDM_FADEIN, OnFadeIn)
	ON_COMMAND(IDM_FADEOUT, OnFadeOut)

	//playback menu items
	ON_COMMAND(IDM_VOLUME0, OnSetVolume0)
	ON_COMMAND(IDM_VOLUME1, OnSetVolume1)
	ON_COMMAND(IDM_VOLUME2, OnSetVolume2)
	ON_COMMAND(IDM_VOLUME3, OnSetVolume3)
	ON_COMMAND(IDM_VOLUME4, OnSetVolume4)
	ON_COMMAND(IDM_VOLUME5, OnSetVolume5)
	ON_COMMAND(IDM_VOLUME6, OnSetVolume6)
	ON_COMMAND(IDM_VOLUME7, OnSetVolume7)
	ON_COMMAND(IDM_VOLUME8, OnSetVolume8)
	ON_COMMAND(IDM_LOOPPLAYBACK, OnLoopPlayback)
	ON_COMMAND(IDM_RIGHTCHANNEL, OnRightChannel)
	ON_COMMAND(IDM_LEFTCHANNEL, OnLeftChannel)
	ON_COMMAND(IDM_INFO, OnInfo)

	//generate menu messages
	ON_COMMAND(IDM_GENERATE, OnGenerate)
	ON_COMMAND(IDM_OVERLAY, OnOverlay)

	//button events
	ON_COMMAND(IDC_PLAY, OnPlay)
	ON_COMMAND(IDC_PAUSE, OnPause)
	ON_COMMAND(IDC_GOTOSTART, OnGotoStart)
	ON_COMMAND(IDC_GOTOEND, OnGotoEnd)
	ON_COMMAND(IDC_FASTFWD, OnFastFwd)
	ON_COMMAND(IDC_REWIND, OnRewind)
	ON_COMMAND(IDC_STOP, OnStop)
	ON_COMMAND(IDC_RECORD, OnRecord)
END_MESSAGE_MAP ()








/*********** Other dialog boxes **********/
//////////////////////////////////////////////////////
//Info Dialog/////////////////////////////////////////
//////////////////////////////////////////////////////

InfoDialog::InfoDialog(char *name) : CDialog(name){
	
	return;
}


BOOL InfoDialog::OnInitDialog() {
	char bu[10];

	CEdit *ed = (CEdit *) GetDlgItem(IDC_FORMATTEXT);
	ed->SetWindowText(mw->ext);
	ed = (CEdit *) GetDlgItem(IDC_CHANNELSTEXT);
	itoa(WaveFormat.nChannels, bu, 10);
	ed->SetWindowText(bu);
	ed = (CEdit *) GetDlgItem(IDC_SAMPLERATETEXT);
	itoa(WaveFormat.nSamplesPerSec, bu, 10);
	ed->SetWindowText(bu);
	ed = (CEdit *) GetDlgItem(IDC_BITSPERSAMPLETEXT);
	itoa(WaveFormat.wBitsPerSample, bu, 10);
	ed->SetWindowText(bu);
	ed = (CEdit *) GetDlgItem(IDC_KBPSTEXT);
	int b = WaveFormat.nAvgBytesPerSec;
	itoa(b, bu, 10);
	ed->SetWindowText(bu);

	return true;
}
BEGIN_MESSAGE_MAP (InfoDialog, CDialog)

END_MESSAGE_MAP()



//////////////////////////////////////////////////////
//Insert Silence//////////////////////////////////////
//////////////////////////////////////////////////////

SilenceDialog::SilenceDialog(char *name) : CDialog(name) {
	return;
}

BOOL SilenceDialog::OnInitDialog() {
	return true;
}

afx_msg void SilenceDialog::OnOK(){
	int x = 0;
	CEdit *e = (CEdit *) GetDlgItem(IDC_SILENCETEXT);
	char bu[10];
	e->GetWindowText((LPTSTR) &bu, 5);
	mw->lensilence = atoi(bu);
	EndDialog(x);
}

BEGIN_MESSAGE_MAP (SilenceDialog, CDialog)
	ON_COMMAND(IDOK, OnOK)
END_MESSAGE_MAP()


//////////////////////////////////////////////////////
//Generate Sound//////////////////////////////////////
//////////////////////////////////////////////////////

GenerateDialog::GenerateDialog(char *name) : CDialog(name) {
	return;
}

int GenerateDialog::GetLength() {
	return length;
}

int GenerateDialog::GetAmplitude() {
	return amplitude;
}

int GenerateDialog::GetFrequency() {
	return freq;
}

BOOL GenerateDialog::OnInitDialog() {
	freq = 0;
	length = 0;
	amplitude = 0;
	return true;
}
afx_msg void GenerateDialog::OnOK() {
	CString cs;
	char b[20];
	CEdit *ed = (CEdit *) GetDlgItem(IDC_FREQLENGTH);
	ed->GetWindowText(b, 10);
	if (strcmp(b, "") == 0) {
		length = 1;
	} else {
		length = atoi(b);
	}
	ed = (CEdit *) GetDlgItem(IDC_FREQ);
	ed->GetWindowText(b, 10);
	if (strcmp(b, "") == 0) {
		freq = 0;
	} else {
		freq = atoi(b);
	}
	ed = (CEdit *) GetDlgItem(IDC_AMPLITUDE);
	ed->GetWindowText(b, 10);
	if (strcmp(b, "") == 0) {
		amplitude = 50;
	} else {
		amplitude = atoi(b);
	}

	if (amplitude > 127) {
		amplitude = 127;
	} 
	if (length < 0) {
		length = 1;
	}
	EndDialog(0);
}

BEGIN_MESSAGE_MAP(GenerateDialog, CDialog)
	ON_COMMAND(IDOK, OnOK)
END_MESSAGE_MAP()



//////////////////////////////////////////////////////
//Overlay Sound //////////////////////////////////////
//////////////////////////////////////////////////////
OverlayDialog::OverlayDialog(char *name) : CDialog(name) {
	return;
}

int OverlayDialog::GetAmplitude() {
	return amplitude;
}

int OverlayDialog::GetFrequency() {
	return freq;
}

BOOL OverlayDialog::OnInitDialog() {
	freq = 0;
	amplitude = 0;
	return true;
}
afx_msg void OverlayDialog::OnOK() {
	CString cs;
	char b[20];
	CEdit * ed;
	ed = (CEdit *) GetDlgItem(IDC_FREQ);
	ed->GetWindowText(b, 10);
	if (strcmp(b, "") == 0) {
		freq = 0;
	} else {
		freq = atoi(b);
	}
	ed = (CEdit *) GetDlgItem(IDC_AMPLITUDE);
	ed->GetWindowText(b, 10);
	if (strcmp(b, "") == 0) {
		amplitude = 50;
	} else {
		amplitude = atoi(b);
	}

	if (amplitude > 127) {
		amplitude = 127;
	} 
	EndDialog(0);
}

BEGIN_MESSAGE_MAP(OverlayDialog, CDialog)
	ON_COMMAND(IDOK, OnOK)
END_MESSAGE_MAP()