/****************************************************************/
/****************************************************************/
/*                 SOUND EDITOR                          
/*
/*			FILE:	soundedit.h
/*			AUTHOR: Ryan C. Clausing
/*			DATE:   Spring 2002
/*
/****************************************************************/
/****************************************************************/

#include <afxcmn.h>
#include <afxdlgs.h>
#include <Afxext.h>
#include <fstream.h>
#include <windows.h>
#include <mmsystem.h>

class CMyApp : public CWinApp
{
public:
	virtual BOOL InitInstance ();
	
};


class InfoDialog : public CDialog {
/********************************************/
/*
/* The class for the file information dialog
/*
/********************************************/
public:
	InfoDialog(char *name);
	BOOL OnInitDialog();
private:
	DECLARE_MESSAGE_MAP();
};


class SilenceDialog : public CDialog {
/********************************************/
/*
/* The class for the insert silence dialog
/*
/********************************************/
public:
	SilenceDialog(char *name);
	BOOL OnInitDialog();

	afx_msg void OnOK();
private:
	DECLARE_MESSAGE_MAP();
};


class GenerateDialog : public CDialog {
/********************************************/
/*
/* The class for the Generate Sound Dialog
/*
/********************************************/
public:
	GenerateDialog(char *name);
	BOOL OnInitDialog();
	int GetLength();
	int GetFrequency();
	int GetAmplitude();
	afx_msg void OnOK();

private:
	int length;
	int freq;
	int amplitude;
	DECLARE_MESSAGE_MAP();
};

class OverlayDialog : public CDialog {
/********************************************/
/*
/* The class for the Overlay Sound Dialog
/*
/********************************************/
public:
	OverlayDialog(char *name);
	BOOL OnInitDialog();
	int GetFrequency();
	int GetAmplitude();
	afx_msg void OnOK();

private:
	int freq;
	int amplitude;
	DECLARE_MESSAGE_MAP();
};


struct Pixel {
/********************************************/
/*
/* The struct for the pixel linked list
/*
/********************************************/
	COLORREF color;
	Pixel *link;
};

struct MyWaveDataNode {
/********************************************/
/*
/* The struct for the wave data to be drawn
/*
/********************************************/
	WORD sample;
	MyWaveDataNode *link;
};

struct MyWaveData {
/********************************************/
/*
/* The struct for the start of the wave data
/* linked lists
/*
/********************************************/
	MyWaveDataNode *rightchannel;
	MyWaveDataNode *leftchannel;
};

struct PureWave {
/********************************************/
/*
/* The struct for defining the pure wave to 
/* be generated
/*
/********************************************/
	int length;
	int freq;
	int amp;
	int count;
	DWORD samplerate;
};

struct MyWaveFile {
/********************************************/
/*
/* The struct for the wave file info and flags
/*
/********************************************/
	long numsamples;
	int bps;
	int max;
	MyWaveData wd;
	CString soundfilepath;
	CString soundfilename;
	CString SoundFileFolder;
	CString modfile, workfile, tmpfile, undofile, clipfile;
	DWORD datasize;
};


class CMainWindow : public CFrameWnd {
public:	
	CMainWindow ();
	void DrawLayout();
	void DrawVerticalLine(int xpos);
	void DrawSelection(float start, float end);
	int GetWaveInfo();
	void FindFolders();

	void LoadWaveData();
	void DrawWaveData();
	void OpenFile();
	void SetVolume();
	void InitVars();

	void RecordStart();
	void RecordEnd();

	void SaveLine();
	void RestoreLine();
	void DrawLine();
	void FakeClose();
	void DeleteWaveDataLists();
	void DeletePosLineList();
	void CalculateFilePositions();
	void ShowStatus();

	//my event handlers
	afx_msg void OnUndo();
	afx_msg void OnSelAll();
	afx_msg void OnSelToEnd();
	afx_msg void OnSelToStart();
	afx_msg void OnSelNone();
	afx_msg void OnSaveToNew();

	afx_msg void OnCrop();
	afx_msg void OnExit();
	afx_msg bool OnCloseFile();
	afx_msg void OnOpen();
	afx_msg void OnNew();
	afx_msg void OnSave();
	afx_msg void OnSaveAs();
	afx_msg void OnCut();
	afx_msg void OnCopy();
	afx_msg void OnPaste();
	afx_msg void OnClear();
	afx_msg void OnInsertSilence();
	afx_msg void OnFadeIn();
	afx_msg void OnFadeOut();

	afx_msg void OnInfo();
	afx_msg void OnPlay();
	afx_msg void OnStop();
	afx_msg void OnPause();
	afx_msg void OnFastFwd();
	afx_msg void OnRewind();
	afx_msg void OnCopyToNew();
	afx_msg void OnPasteToNew();
	afx_msg void OnReverse();
	afx_msg void OnSwapChannels();
	afx_msg void OnOverlay();
	
	afx_msg void OnGotoStart();
	afx_msg void OnGotoEnd();

	afx_msg void OnRecord();
	afx_msg void OnVolumeUp();
	afx_msg void OnVolumeDown();
	afx_msg void OnRevert();

	afx_msg void OnSetVolume0();
	afx_msg void OnSetVolume1();
	afx_msg void OnSetVolume2();
	afx_msg void OnSetVolume3();
	afx_msg void OnSetVolume4();
	afx_msg void OnSetVolume5();
	afx_msg void OnSetVolume6();
	afx_msg void OnSetVolume7();
	afx_msg void OnSetVolume8();

	afx_msg void OnRightChannel();
	afx_msg void OnLeftChannel();

	afx_msg void OnLoopPlayback();

	afx_msg void OnGenerate();

	// menu update things
	afx_msg void OnSetMenus(CCmdUI *pCmdUI);

	//windows events
	afx_msg int OnCreate(LPCREATESTRUCT str);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnClose();
	afx_msg void OnSetFocus(CWnd *pOldWnd);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);


	//public variables 
	bool lbuttondown, working;
	bool paused, stopped, playing, fastfwd, rewind, fileopen, processing, changedsincesaved, loop, recording;
	WORD wDeviceID;
	DWORD dwThreadID;
	HANDLE hThread;
	int volumelevel;
	Pixel *posline;

	CString windowstempdir;
	DWORD mci_wavelength, PlayStart, PlayEnd;
	float linepos;
	float startselection, endselection;
	unsigned long lastmark, nextmark;
	int zeroval;
	CRect winrect, toprect, topline, bottomline, tophalf, bottomhalf, statusbox;
	int qlinetop, qlinebottom, midline;
	int lensilence;
	CString ext;
	DWORD FileDataStartPos, CurrentFileDataPos, FileSelStart, FileSelEnd, SizeOfClipboardData;
	CString statusstate, laststatusstate, statustext, laststatustext;
	CRect arect;
	int overlays;
	char buff[100];
	CToolBarCtrl ToolBar;
	MyWaveFile wavefile;
	PureWave pw;

private:
	bool shiftkey, controlkey;
	bool rightchannel, leftchannel;
	COLORREF GrayColor;
	DWORD dwVolume;
	HICON PlayIcon, FastFwdIcon, RewindIcon, PauseIcon, RecordIcon, StopIcon, JumpToStartIcon, JumpToEndIcon;
	DWORD playstart, playend;
	CImageList EnabledImageList, DisabledImageList, HotImageList;
	TBBUTTON AButton;


protected:
	virtual void PostNcDestroy ();

    DECLARE_MESSAGE_MAP ()
};


